@@ -9,7 +9,7 @@ from __future__ import annotations
 
 import logging
 from types import SimpleNamespace
-from typing import Any, Optional, TypeVar
+from typing import Any, Callable, Optional
 
 from ..const import DEV_TYPE, DEV_TYPE_MAP, SZ_DEVICE_ID, __dev_mode__
 from ..entity_base import Child, Entity, class_by_attr
@@ -66,19 +66,16 @@ if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
-_DeviceT = TypeVar("_DeviceT", bound="Device")
-
-
 def check_faking_enabled(fnc):
     def wrapper(self, *args, **kwargs):
         if not self._faked:
             raise RuntimeError(f"Faking is not enabled for {self}")
-        return fnc(*args, **kwargs)
+        return fnc(self, *args, **kwargs)
 
     return wrapper
 
 
-class Device(Entity):
+class DeviceBase(Entity):
     """The Device base class - can also be used for unknown device types."""
 
     _SLUG: str = DEV_TYPE.DEV  # type: ignore[assignment]
@@ -215,10 +212,11 @@ class Device(Entity):
     @property
     def traits(self) -> dict[str, Any]:
         """Return the traits of the device."""
-        known_dev = self._gwy._include.get(self.id)
 
         result = super().traits
 
+        known_dev = self._gwy._include.get(self.id)
+
         result.update(
             {
                 SZ_CLASS: DEV_TYPE_MAP[self._SLUG],
@@ -227,166 +225,10 @@ class Device(Entity):
             }
         )
 
-        if Code._10E0 in self._msgs or Code._10E0 in CODES_BY_DEV_SLUG.get(
-            self._SLUG, []
-        ):
-            result.update({"_info": self.device_info})
-
-        result.update({"_bind": self._msg_value(Code._1FC9)})
-
-        return result
+        return result | {"_bind": self._msg_value(Code._1FC9)}
 
 
-class DeviceInfo(Device):  # 10E0
-    def _setup_discovery_tasks(self) -> None:
-        super()._setup_discovery_tasks()
-
-        # if discover_flag & Discover.SCHEMA:
-        if self._SLUG not in CODES_BY_DEV_SLUG or RP in CODES_BY_DEV_SLUG[
-            self._SLUG
-        ].get(Code._10E0, {}):
-            self._add_discovery_task(
-                _mk_cmd(RQ, Code._10E0, "00", self.id), 60 * 60 * 24
-            )
-
-    @property
-    def device_info(self) -> Optional[dict]:  # 10E0
-        return self._msg_value(Code._10E0)
-
-
-class Fakeable(Device):
-    def __init__(self, gwy, *args, **kwargs) -> None:
-        super().__init__(gwy, *args, **kwargs)
-
-        self._faked: bool = None  # type: ignore[assignment]
-
-        self._1fc9_state = {"state": BindState.UNKNOWN}
-
-        if self.id in gwy._include and gwy._include[self.id].get(SZ_FAKED):
-            self._make_fake()
-
-        if kwargs.get(SZ_FAKED):
-            self._make_fake()
-
-    @check_faking_enabled
-    def _bind(self):
-        self._1fc9_state["state"] = BindState.UNBOUND
-
-    def _make_fake(self, bind=None) -> Device:
-        if not self._faked:
-            self._faked = True
-            self._gwy._include[self.id] = {SZ_FAKED: True}
-            _LOGGER.warning(f"Faking now enabled for {self}")  # TODO: be info/debug
-        if bind:
-            self._bind()
-        return self
-
-    def _bind_waiting(self, code, idx="00", callback=None):
-        """Wait for (listen for) a bind handshake."""
-
-        # Bind waiting: BDR set to listen, CTL initiates handshake
-        # 19:30:44.749 051  I --- 01:054173 --:------ 01:054173 1FC9 024 FC-0008-04D39D FC-3150-04D39D FB-3150-04D39D FC-1FC9-04D39D
-        # 19:30:45.342 053  W --- 13:049798 01:054173 --:------ 1FC9 012 00-3EF0-34C286 00-3B00-34C286
-        # 19:30:45.504 049  I --- 01:054173 13:049798 --:------ 1FC9 006 00-FFFF-04D39D
-
-        # Bind waiting: OTB set to listen, CTL initiates handshake
-        # 00:25:02.779 045  I --- 01:145038 --:------ 01:145038 1FC9 024 FC-0008-06368E FC-3150-06368E FB-3150-06368E  FC-1FC9-06368E  # opentherm bridge
-        # 00:25:02.792 045  W --- 10:048122 01:145038 --:------ 1FC9 006 00-3EF0-28BBFA
-        # 00:25:02.944 045  I --- 01:145038 10:048122 --:------ 1FC9 006 00-FFFF-06368E
-
-        _LOGGER.warning(f"Binding {self}: waiting for {code} for 300 secs")  # info
-        # SUPPORTED_CODES = (Code._0008,)
-
-        def proc_confirm(msg, *args) -> None:
-            """Process the 3rd/final packet of the handshake."""
-            if not msg or msg.code != code:
-                return
-
-            self._1fc9_state["state"] = BindState.BOUND
-            if callback:
-                callback(msg)
-
-        def proc_offer(msg, *args) -> None:
-            """Process the 1st, and send the 2nd, packet of the handshake."""
-            if not msg:
-                return
-            # assert code in SUPPORTED_CODES, f"Binding {self}: {code} is not supported"
-            self._1fc9_state["msg"] = msg  # the offer
-
-            self._1fc9_state["state"] = BindState.ACCEPTING
-            cmd = Command.put_bind(
-                W_,
-                code,
-                self.id,
-                idx=idx,  # zone_idx or domain_id
-                dst_id=msg.src.id,
-                callback={
-                    SZ_FUNC: proc_confirm,
-                    SZ_TIMEOUT: 3,
-                },  # re-Tx W until Rx an I
-            )
-            self._send_cmd(cmd)
-
-        self._1fc9_state["code"] = code
-        self._1fc9_state["state"] = BindState.LISTENING
-        self._gwy.msg_transport._add_callback(
-            f"{Code._1FC9}|{I_}|{NUL_DEV_ADDR.id}",
-            {SZ_FUNC: proc_offer, SZ_TIMEOUT: 300},
-        )
-
-    def _bind_request(self, code, callback=None):
-        """Initiate a bind handshake: send the 1st packet of the handshake."""
-
-        # Bind request: CTL set to listen, STA initiates handshake (note 3C09/2309)
-        # 22:13:52.527 070  I --- 34:021943 --:------ 34:021943 1FC9 024 00-2309-8855B7 00-30C9-8855B7 00-0008-8855B7 00-1FC9-8855B7
-        # 22:13:52.540 052  W --- 01:145038 34:021943 --:------ 1FC9 006 00-2309-06368E
-        # 22:13:52.572 071  I --- 34:021943 01:145038 --:------ 1FC9 006 00-2309-8855B7
-
-        # Bind request: CTL set to listen, DHW sensor initiates handshake
-        # 19:45:16.733 045  I --- 07:045960 --:------ 07:045960 1FC9 012 00-1260-1CB388 00-1FC9-1CB388
-        # 19:45:16.896 045  W --- 01:054173 07:045960 --:------ 1FC9 006 00-10A0-04D39D
-        # 19:45:16.919 045  I --- 07:045960 01:054173 --:------ 1FC9 006 00-1260-1CB388
-
-        _LOGGER.warning(f"Binding {self}: requesting {code}")  # TODO: info
-        SUPPORTED_CODES = (Code._0002, Code._1260, Code._1290, Code._30C9)
-
-        def proc_accept(msg, *args) -> None:
-            """Process the 2nd, and send the 3rd/final, packet of the handshake."""
-            if not msg or msg.dst is not self:
-                return
-
-            self._1fc9_state["msg"] = msg  # the accept
-
-            self._1fc9_state["state"] = BindState.CONFIRMING
-            cmd = Command.put_bind(I_, code, self.id, dst_id=msg.src.id)
-            self._send_cmd(cmd)
-
-            self._1fc9_state["state"] = BindState.BOUND
-            if callback:
-                callback(msg)
-
-        assert code in SUPPORTED_CODES, f"Binding {self}: {code} is not supported"
-
-        self._1fc9_state["code"] = code
-        self._1fc9_state["state"] = BindState.OFFERING
-        cmd = Command.put_bind(
-            I_, code, self.id, callback={SZ_FUNC: proc_accept, SZ_TIMEOUT: 3}
-        )
-        self._send_cmd(cmd)
-
-    @property
-    def is_faked(self) -> bool:
-        return bool(self._faked)
-
-    @property
-    def traits(self) -> dict:
-
-        result = super().traits
-        result[SZ_FAKED] = self.is_faked
-        return result
-
-
-class BatteryState(Device):  # 1060
+class BatteryState(DeviceBase):  # 1060
 
     BATTERY_LOW = "battery_low"  # boolean
     BATTERY_STATE = "battery_state"  # percentage (0.0-1.0)
@@ -411,6 +253,183 @@ class BatteryState(Device):  # 1060
         }
 
 
+class DeviceInfo(DeviceBase):  # 10E0
+    def _setup_discovery_tasks(self) -> None:
+        super()._setup_discovery_tasks()
+
+        # if discover_flag & Discover.SCHEMA:
+        if self._SLUG not in CODES_BY_DEV_SLUG or RP in CODES_BY_DEV_SLUG[
+            self._SLUG
+        ].get(Code._10E0, {}):
+            self._add_discovery_task(
+                _mk_cmd(RQ, Code._10E0, "00", self.id), 60 * 60 * 24
+            )
+
+    @property
+    def device_info(self) -> Optional[dict]:  # 10E0
+        return self._msg_value(Code._10E0)
+
+    @property
+    def traits(self) -> dict[str, Any]:
+        """Return the traits of the device."""
+
+        result = super().traits
+
+        if Code._10E0 in self._msgs or Code._10E0 in CODES_BY_DEV_SLUG.get(
+            self._SLUG, []
+        ):
+            result.update({"_info": self.device_info})
+
+        return result
+
+
+class Fakeable(DeviceBase):
+
+    # Faked Round Thermostat binding to a Evohome controller as a zone sensor
+    # STA set to BindState.OFFERING:  sends "1FC9| I|63:262142" to the ether (the controller is listening)
+    # - receives "1FC9| W|34:021943" to: Fakable._bind_request().proc_accept() via callback?
+    # STA set to BindState.ACCEPTING: sends "1FC9| I|01:145038" to the ether (the controller is ignoring?)
+    # - receives "1FC9| I|01:145038" to: Fakable._bind_request().proc_confirm() via callback -- OPTIONAL!!
+    # STA set to BindState.BOUND via callback, and...
+
+    # Faked Evohome controller binding to a Round Thermostat as a zone sensor
+    # CTL set to BindState.LISTENING
+    # - receives "1FC9| I|63:262142" to: Fakable._bind_waiting().proc_offer() via callback?
+    # CTL set to BindState.ACCEPTING: sends "1FC9| W|34:021943" to the ether
+    # - receives "1FC9| I|01:145038" to: Fakable._bind_waiting().proc_confirm() via callback -- OPTIONAL!!
+    # CTL set to BindState.BOUND
+
+    def __init__(self, gwy, *args, **kwargs) -> None:
+        super().__init__(gwy, *args, **kwargs)
+
+        self._faked: bool = None  # type: ignore[assignment]
+
+        self._1fc9_state = {"state": BindState.UNKNOWN}
+
+        if self.id in gwy._include and gwy._include[self.id].get(SZ_FAKED):
+            self._make_fake()
+
+        if kwargs.get(SZ_FAKED):
+            self._make_fake()
+
+    @check_faking_enabled
+    def _bind(self):
+        self._1fc9_state["state"] = BindState.UNBOUND
+
+    def _make_fake(self, bind=None) -> Fakeable:
+        if not self._faked:
+            self._faked = True
+            self._gwy._include[self.id] = {SZ_FAKED: True}
+            _LOGGER.warning(f"Faking now enabled for {self}")  # TODO: be info/debug
+        if bind:
+            self._bind()
+        return self
+
+    def _bind_waiting(self, codes, idx="00", callback=None):
+        """Wait for (listen for) a bind handshake."""
+
+        # Bind waiting: BDR set to listen, CTL initiates handshake
+        # 19:30:44.749 051  I --- 01:054173 --:------ 01:054173 1FC9 024 FC-0008-04D39D FC-3150-04D39D FB-3150-04D39D FC-1FC9-04D39D
+        # 19:30:45.342 053  W --- 13:049798 01:054173 --:------ 1FC9 012 00-3EF0-34C286 00-3B00-34C286
+        # 19:30:45.504 049  I --- 01:054173 13:049798 --:------ 1FC9 006 00-FFFF-04D39D
+
+        # Bind waiting: OTB set to listen, CTL initiates handshake
+        # 00:25:02.779 045  I --- 01:145038 --:------ 01:145038 1FC9 024 FC-0008-06368E FC-3150-06368E FB-3150-06368E  FC-1FC9-06368E  # opentherm bridge
+        # 00:25:02.792 045  W --- 10:048122 01:145038 --:------ 1FC9 006 00-3EF0-28BBFA
+        # 00:25:02.944 045  I --- 01:145038 10:048122 --:------ 1FC9 006 00-FFFF-06368E
+
+        _LOGGER.warning(f"Binding {self}: waiting for {codes} for 300 secs")  # info
+        # SUPPORTED_CODES = (Code._0008,)
+
+        def proc_confirm(msg, *args) -> None:
+            """Process the 3rd/final packet of the handshake."""
+            # if not msg or msg.code not in codes:
+            #     return
+
+            self._1fc9_state["state"] = BindState.BOUND
+            if callback:
+                callback(msg)
+
+        def proc_offer(msg, *args) -> None:
+            """Process the 1st, and send the 2nd, packet of the handshake."""
+            if not msg:
+                return
+            # assert code in SUPPORTED_CODES, f"Binding {self}: {code} is not supported"
+            self._1fc9_state["msg"] = msg  # the offer
+
+            self._1fc9_state["state"] = BindState.ACCEPTING
+            cmd = Command.put_bind(
+                W_,
+                codes,
+                self.id,
+                idx=idx,  # zone_idx or domain_id
+                dst_id=msg.src.id,
+                callback={
+                    SZ_FUNC: proc_confirm,
+                    SZ_TIMEOUT: 3,
+                },  # re-Tx W until Rx an I
+            )
+            self._send_cmd(cmd)
+
+        self._1fc9_state["codes"] = codes
+        self._1fc9_state["state"] = BindState.LISTENING
+        self._gwy.msg_transport._add_callback(
+            f"{Code._1FC9}|{I_}|{NUL_DEV_ADDR.id}",
+            {SZ_FUNC: proc_offer, SZ_TIMEOUT: 300},
+        )
+
+    def _bind_request(self, codes, callback: Callable = None) -> None:
+        """Initiate a bind handshake: send the 1st packet of the handshake."""
+
+        # Bind request: CTL set to listen, STA initiates handshake (note 3C09/2309)
+        # 22:13:52.527 070  I --- 34:021943 --:------ 34:021943 1FC9 024 00-2309-8855B7 00-30C9-8855B7 00-0008-8855B7 00-1FC9-8855B7
+        # 22:13:52.540 052  W --- 01:145038 34:021943 --:------ 1FC9 006 00-2309-06368E
+        # 22:13:52.572 071  I --- 34:021943 01:145038 --:------ 1FC9 006 00-2309-8855B7
+
+        # Bind request: CTL set to listen, DHW sensor initiates handshake
+        # 19:45:16.733 045  I --- 07:045960 --:------ 07:045960 1FC9 012 00-1260-1CB388 00-1FC9-1CB388
+        # 19:45:16.896 045  W --- 01:054173 07:045960 --:------ 1FC9 006 00-10A0-04D39D
+        # 19:45:16.919 045  I --- 07:045960 01:054173 --:------ 1FC9 006 00-1260-1CB388
+
+        _LOGGER.warning(f"Binding {self}: requesting {codes}")  # TODO: info
+
+        def proc_accept(msg, *args) -> None:
+            """Process the 2nd, and send the 3rd/final, packet of the handshake."""
+            if not msg or msg.dst is not self:
+                return
+
+            self._1fc9_state["msg"] = msg  # the accept
+
+            self._1fc9_state["state"] = BindState.CONFIRMING
+            cmd = Command.put_bind(I_, codes, self.id, dst_id=msg.src.id)
+            self._send_cmd(cmd)
+
+            self._1fc9_state["state"] = BindState.BOUND
+            if callback:
+                callback(msg)
+
+        if not isinstance(codes, tuple):
+            codes = (codes,)
+
+        self._1fc9_state["codes"] = codes
+        self._1fc9_state["state"] = BindState.OFFERING
+        cmd = Command.put_bind(
+            I_, codes, self.id, callback={SZ_FUNC: proc_accept, SZ_TIMEOUT: 3}
+        )
+        self._send_cmd(cmd)
+
+    @property
+    def is_faked(self) -> bool:
+        return bool(self._faked)
+
+    @property
+    def traits(self) -> dict:
+
+        result = super().traits
+        result[SZ_FAKED] = self.is_faked
+        return result
+
+
 class HgiGateway(DeviceInfo):  # HGI (18:)
     """The HGI80 base class."""
 
@@ -457,20 +476,22 @@ class HgiGateway(DeviceInfo):  # HGI (18:)
         if device_id[:2] != dev_type:
             raise TypeError(f"Invalid device ID {device_id} for type '{dev_type}:'")
 
-        dev = self.device_by_id.get(device_id)
-        if dev:  # TODO: BUG: is broken
-            _LOGGER.warning("Destroying %s", dev)
-            if dev._parent:
-                del dev._parent.child_by_id[dev.id]
-                dev._parent.childs.remove(dev)
-                dev._parent = None
-            del self.device_by_id[dev.id]
-            self.devices.remove(dev)
-            dev = None
+        # dev = self.device_by_id.get(device_id)
+        # if dev:  # TODO: BUG: is broken
+        #     _LOGGER.warning("Destroying %s", dev)
+        #     if dev._parent:
+        #         del dev._parent.child_by_id[dev.id]
+        #         dev._parent.childs.remove(dev)
+        #         dev._parent = None
+        #     del self.device_by_id[dev.id]
+        #     self.devices.remove(dev)
+        #     dev = None
 
-        dev = self._gwy.get_device(device_id)
-        dev._make_fake(bind=True)
-        return dev
+        # dev = self._gwy.get_device(device_id)
+        # dev._make_fake(bind=True)
+        # return dev
+
+        raise NotImplementedError
 
     def create_fake_bdr(self, device_id=DEFAULT_BDR_ID) -> Device:
         """Bind a faked relay (BDR91A) to a controller (i.e. to a domain/zone).
@@ -532,9 +553,11 @@ class HgiGateway(DeviceInfo):  # HGI (18:)
         }
 
 
-class DeviceHeat(
-    Child, DeviceInfo
-):  # Honeywell CH/DHW or compatible (incl. UFH, Heatpumps)
+class Device(Child, DeviceBase):
+    pass
+
+
+class DeviceHeat(Device):  # Honeywell CH/DHW or compatible
     """The base class for Honeywell CH/DHW-compatible devices.
 
     Includes UFH and heatpumps (which can also cool).
@@ -596,7 +619,7 @@ class DeviceHeat(
         return self._parent
 
 
-class DeviceHvac(Child, DeviceInfo):  # HVAC (ventilation, PIV, MV/HR)
+class DeviceHvac(Device):  # HVAC (ventilation, PIV, MV/HR)
     """The Device base class for HVAC (ventilation, PIV, MV/HR)."""
 
     _SLUG: str = DEV_TYPE.HVC  # these may be instantiated, and promoted later on
