@@ -5,23 +5,27 @@
 
 The serial to RF gateway (HGI80, not RFG100).
 """
+from __future__ import annotations
 
 import asyncio
 import logging
 import os
 import signal
-from asyncio.futures import Future
 from concurrent import futures
 from datetime import datetime as dt
 from threading import Lock
-from typing import Callable, Optional
+from types import SimpleNamespace
+from typing import Callable, Optional, TextIO
 
-from ramses_rf.devices_base import DeviceHeat, DeviceHvac
+from ramses_rf.device import DeviceHeat, DeviceHvac, Fakeable
+from ramses_rf.protocol.frame import _CodeT, _DeviceIdT, _PayloadT, _VerbT
+from ramses_rf.protocol.protocol import MessageProtocol, MessageTransport
+from ramses_rf.protocol.transport import PacketProtocolBase
 
 from .const import DONT_CREATE_MESSAGES, SZ_DEVICE_ID, SZ_DEVICES, __dev_mode__
-from .devices import Device, zx_device_factory
+from .device import Device, zx_device_factory
 from .helpers import schedule_task, shrink
-from .message import Message, process_msg
+from .processor import Message, process_msg
 from .protocol import (
     SZ_POLLER_TASK,
     Address,
@@ -33,15 +37,14 @@ from .protocol import (
     set_pkt_logging_config,
 )
 from .protocol.address import HGI_DEV_ADDR, NON_DEV_ADDR, NUL_DEV_ADDR
-from .schema import (
-    DEBUG_MODE,
-    ENFORCE_KNOWN_LIST,
-    INPUT_FILE,
-    SCHEMA_DEV,
+from .schemas import (
+    SCH_GLOBAL_GATEWAY_CONFIG,
+    SCH_TRAITS,
     SZ_ALIAS,
     SZ_BLOCK_LIST,
     SZ_CLASS,
     SZ_CONFIG,
+    SZ_ENFORCE_KNOWN_LIST,
     SZ_FAKED,
     SZ_KNOWN_LIST,
     SZ_MAIN_CONTROLLER,
@@ -49,7 +52,7 @@ from .schema import (
     load_config,
     load_schema,
 )
-from .systems import System
+from .system import System
 
 # skipcq: PY-W2000
 from .protocol import (  # noqa: F401, isort: skip, pylint: disable=unused-import
@@ -61,7 +64,7 @@ from .protocol import (  # noqa: F401, isort: skip, pylint: disable=unused-impor
     FA,
     FC,
     FF,
-    _313F,
+    Code,
 )
 
 
@@ -75,21 +78,43 @@ if DEV_MODE:
 class Engine:
     """The engine class."""
 
-    def __init__(self, serial_port, loop=None) -> None:
+    _create_msg_stack: Callable = create_msg_stack
+    _create_pkt_stack: Callable = create_pkt_stack
 
-        self._loop = loop or asyncio.get_running_loop()
+    def __init__(
+        self,
+        serial_port: None | str,
+        input_file: None | TextIO = None,
+        port_config: None | dict = None,
+        loop: None | asyncio.AbstractEventLoop = None,
+    ) -> None:
 
         self.ser_name = serial_port
-        self._input_file = None
+        self._input_file = input_file
+        self._port_config = port_config or {}
+        self._loop = loop or asyncio.get_running_loop()
 
-        self.pkt_protocol, self.pkt_transport = None, None
-        self.msg_protocol, self.msg_transport = None, None
+        self._include: dict[_DeviceIdT, dict] = {}  # aka known_list, and ?allow_list
+        self._exclude: dict[_DeviceIdT, dict] = {}  # aka block_list
+        self._unwanted: list[_DeviceIdT] = [
+            NON_DEV_ADDR.id,
+            NUL_DEV_ADDR.id,
+            "01:000001",
+        ]
+
+        self.config = SimpleNamespace()  # **SCH_CONFIG_GATEWAY({}))
+
+        self.msg_protocol: MessageProtocol = None  # type: ignore[assignment]
+        self.msg_transport: MessageTransport = None  # type: ignore[assignment]
+        self.pkt_protocol: PacketProtocolBase = None  # type: ignore[assignment]
+        self.pkt_transport: asyncio.Transport = None  # type: ignore[assignment]
 
         self._engine_lock = Lock()
-        self._engine_state = None
+        self._engine_state: None | tuple[None | Callable, tuple] = None
 
     def __str__(self) -> str:
-        return (self.hgi or HGI_DEV_ADDR).id + f" ({self.ser_name})"
+        hgi_id = self.pkt_protocol._hgi80[SZ_DEVICE_ID] if self.pkt_protocol else ""
+        return (hgi_id or HGI_DEV_ADDR.id) + f" ({self.ser_name})"
 
     def _setup_event_handlers(self) -> None:  # HACK: for dev/test only
         def handle_exception(loop, context):
@@ -108,9 +133,9 @@ class Engine:
         # return dt.now()
         return self.pkt_protocol._dt_now() if self.pkt_protocol else dt.now()
 
-    def create_client(self, msg_handler) -> tuple[Callable, Callable]:
+    def create_client(self, msg_handler: Callable) -> tuple:
         """Create a client protocol for the RAMSES-II message transport."""
-        return create_msg_stack(self, msg_handler)
+        return self._create_msg_stack(msg_handler)
 
     async def start(self) -> None:
         self._start()
@@ -120,27 +145,24 @@ class Engine:
 
         (_LOGGER.warning if DEV_MODE else _LOGGER.debug)("ENGINE: Starting poller...")
 
-        if self.ser_name:  # source of packets is a serial port
-            pkt_receiver = (
-                self.msg_transport.get_extra_info(self.msg_transport.READER)
-                if self.msg_transport
-                else None
-            )
-            self.pkt_protocol, self.pkt_transport = create_pkt_stack(
-                self, pkt_receiver, ser_port=self.ser_name
-            )  # TODO: can raise SerialException
-            if self.msg_transport:
-                self.msg_transport._set_dispatcher(self.pkt_protocol.send_data)
+        pkt_receiver = (
+            self.msg_transport.get_extra_info(self.msg_transport.READER)
+            if self.msg_transport
+            else None
+        )
 
+        if self.ser_name:
+            source = {"ser_port": self.ser_name}
+        else:
+            source = {"packet_log": self._input_file}
+
+        self.pkt_protocol, self.pkt_transport = self._create_pkt_stack(
+            pkt_receiver, **source
+        )  # TODO: may raise SerialException
+
+        if self.ser_name:  # and self.msg_transport:
+            self.msg_transport._set_dispatcher(self.pkt_protocol.send_data)
         else:  # if self._input_file:
-            pkt_receiver = (
-                self.msg_transport.get_extra_info(self.msg_transport.READER)
-                if self.msg_transport
-                else None
-            )
-            self.pkt_protocol, self.pkt_transport = create_pkt_stack(
-                self, pkt_receiver, packet_log=self._input_file
-            )
             set_logger_timesource(self.pkt_protocol._dt_now)
             _LOGGER.warning("Datetimes maintained as most recent packet log timestamp")
 
@@ -174,7 +196,7 @@ class Engine:
             self._engine_lock.release()
             raise RuntimeError("Unable to pause engine, it is already paused")
 
-        self._engine_state, callback = (None, None), None
+        self._engine_state, callback = (None, tuple()), None
         self._engine_lock.release()
 
         if self.pkt_protocol:
@@ -198,7 +220,10 @@ class Engine:
             self._engine_lock.release()
             raise RuntimeError("Unable to resume engine, it was not paused")
 
+        callback: None | Callable
+        args: tuple
         callback, args = self._engine_state
+
         self._engine_lock.release()
 
         if self.pkt_protocol:
@@ -210,33 +235,21 @@ class Engine:
         return args
 
     @property
-    def hgi(self) -> Optional[Device]:
-        """Return the HGI80-compatible gateway device."""
-
-        if self.pkt_protocol and self.pkt_protocol._hgi80[SZ_DEVICE_ID]:
-            return self.device_by_id.get(self.pkt_protocol._hgi80[SZ_DEVICE_ID])
-
-    @property
-    def pkt_source(self) -> asyncio.Task:
+    def pkt_source(self) -> None | asyncio.Task:
         if t := self.msg_transport:
             return t.get_extra_info(t.WRITER)
+        return None
 
     @staticmethod
-    def create_cmd(verb, device_id, code, payload, **kwargs) -> Command:
+    def create_cmd(
+        verb: _VerbT, device_id: _DeviceIdT, code: _CodeT, payload: _PayloadT, **kwargs
+    ) -> Command:
         """Make a command addressed to device_id."""
+        return Command.from_attrs(verb, device_id, code, payload, **kwargs)
 
-        try:
-            return Command(verb, code, payload, device_id)
-        except (
-            AssertionError,
-            AttributeError,
-            LookupError,
-            TypeError,
-            ValueError,
-        ) as exc:
-            _LOGGER.exception(f"create_cmd(): {exc}")
-
-    def send_cmd(self, cmd: Command, callback: Callable = None, **kwargs) -> Future:
+    def send_cmd(
+        self, cmd: Command, callback: Callable = None, **kwargs
+    ) -> futures.Future[Optional[Message]]:
         """Send a command with the option to return any response message via callback.
 
         Response packets, if any (an RP/I will follow an RQ/W), and have the same code.
@@ -251,7 +264,7 @@ class Engine:
             self._loop,
         )
 
-    async def async_send_cmd(self, cmd: Command, **kwargs) -> Message:
+    async def async_send_cmd(self, cmd: Command, **kwargs) -> None | Message:
         """Send a command with the option to not wait for a response message.
 
         Response packets, if any, follow an RQ/W (as an RP/I), and have the same code.
@@ -291,46 +304,51 @@ class Engine:
 class Gateway(Engine):
     """The gateway class."""
 
-    def __init__(self, serial_port, loop=None, **kwargs) -> None:
+    def __init__(
+        self,
+        serial_port: None | str,
+        debug_mode: None | bool = None,
+        input_file: None | TextIO = None,
+        port_config: None | dict = None,
+        loop: None | asyncio.AbstractEventLoop = None,
+        **kwargs,
+    ) -> None:
 
-        if kwargs.pop(DEBUG_MODE, None):
+        if debug_mode:
             _LOGGER.setLevel(logging.DEBUG)  # should be INFO?
         _LOGGER.debug("Starting RAMSES RF, **kwargs = %s", kwargs)
 
-        super().__init__(serial_port, loop=loop)
+        super().__init__(
+            serial_port, input_file=input_file, port_config=port_config, loop=loop
+        )
 
-        self._tasks: list = []
-
-        self._input_file = kwargs.pop(INPUT_FILE, None)
-
-        self._include: dict = {}  # the provided known_list (?and used as an allow_list)
-        self._exclude: dict = {}  # the provided block_list
+        self._tasks: list = []  # TODO: used by discovery, move lower?
+        self._schema: dict[str, dict] = {}
 
         (self.config, self._schema, self._include, self._exclude) = load_config(
-            self.ser_name, self._input_file, **kwargs
+            self.ser_name,
+            self._input_file,
+            **SCH_GLOBAL_GATEWAY_CONFIG(
+                {k: v for k, v in kwargs.items() if k[:1] != "_"}
+            ),
         )
-        self._unwanted = [NON_DEV_ADDR.id, NUL_DEV_ADDR.id, "01:000001"]
-
         set_pkt_logging_config(
             cc_console=self.config.reduce_processing >= DONT_CREATE_MESSAGES,
-            **self.config.packet_log,
+            **self.config.packet_log or {},
         )
 
         if self.config.reduce_processing < DONT_CREATE_MESSAGES:
             self.msg_protocol, self.msg_transport = self.create_client(process_msg)
 
         # if self.config.reduce_processing > 0:
-        self._tcs: System = None
+        self._tcs: None | System = None  # type: ignore[assignment]
         self.devices: list[Device] = []
-        self.device_by_id: dict = {}
+        self.device_by_id: dict[str, Device] = {}
 
         self._setup_event_handlers()
 
         load_schema(self, **self._schema)
 
-        self._db = None
-        self._cur = None
-
     def __repr__(self) -> str:
         return super().__str__()
 
@@ -363,7 +381,7 @@ class Gateway(Engine):
         else:  # unsupported
             raise RuntimeError(f"Unsupported OS for this module: {os.name}")
 
-    async def start(self, *, start_discovery=True) -> None:
+    async def start(self, *, start_discovery: bool = True) -> None:
         def initiate_discovery(dev_list, sys_list) -> None:
             _LOGGER.debug("ENGINE: Initiating/enabling discovery...")
 
@@ -391,14 +409,14 @@ class Gateway(Engine):
         # if self._engine_state is None:
         #     self._pause()
 
-        if tasks := [t.cancel() for t in self._tasks if not t.done()]:
+        if [t.cancel() for t in self._tasks if not t.done()]:
             try:  # TODO: except asyncio.CancelledError:
-                await asyncio.gather(*tasks)
+                await asyncio.gather(*self._tasks)
             except TypeError:  # HACK
                 pass
         await super().stop()
 
-    def _pause(self, *args, clear_state=False) -> None:
+    def _pause(self, *args, clear_state: bool = False) -> None:
         """Pause the (unpaused) gateway."""
 
         super()._pause(
@@ -419,7 +437,7 @@ class Gateway(Engine):
             *args,
         ) = super()._resume()
 
-        return args
+        return args  # type: ignore[return-value]
 
     def _clear_state(self) -> None:
         _LOGGER.warning("ENGINE: Clearing exisiting schema/state...")
@@ -428,7 +446,7 @@ class Gateway(Engine):
         self.devices = []
         self.device_by_id = {}
 
-    def get_state(self, include_expired=None) -> tuple[dict, dict]:
+    def get_state(self, include_expired: bool = False) -> tuple[dict, dict]:
 
         (_LOGGER.warning if DEV_MODE else _LOGGER.debug)("ENGINE: Getting state...")
         self._pause()
@@ -440,7 +458,14 @@ class Gateway(Engine):
 
         return result
 
-    def _get_state(self, include_expired=None) -> tuple[dict, dict]:
+    def _get_state(self, include_expired: bool = False) -> tuple[dict, dict]:
+        def wanted_msg(msg: Message) -> bool:
+            # 313F will usu. be expired, but will be useful for back-back restarts
+            if msg.code == Code._313F:
+                return True
+            # if msg.code == Code._1FC9 and msg.verb != RP:
+            #     return True
+            return not msg._expired
 
         msgs = [m for device in self.devices for m in device._msg_db]
 
@@ -453,15 +478,12 @@ class Gateway(Engine):
         pkts = {
             f"{repr(msg._pkt)[:26]}": f"{repr(msg._pkt)[27:]}"
             for msg in msgs
-            if msg.verb in (I_, RP)
-            and (
-                include_expired or msg.code == _313F or not msg._expired
-            )  # 313F will be expired, but will be useful for back-back restarts
+            if msg.verb in (I_, RP) and include_expired or wanted_msg(msg)
         }  # BUG: assumes pkts have unique dtms
 
         return self.schema, dict(sorted(pkts.items()))
 
-    async def set_state(self, packets, *, schema=None) -> None:
+    async def set_state(self, packets, *, schema: None | dict = None) -> None:
         (_LOGGER.warning if DEV_MODE else _LOGGER.info)("ENGINE: Setting state...")
 
         if schema is None:  # TODO: also for known_list (device traits)?
@@ -470,26 +492,33 @@ class Gateway(Engine):
         self._pause(clear_state=True)
 
         load_schema(self, **schema)
-        self._set_state(packets, schema=schema)
+        await self._set_state(packets, schema=schema)
 
         self._resume()
         (_LOGGER.warning if DEV_MODE else _LOGGER.info)("ENGINE: Set state.")
 
     async def _set_state(self, packets: dict, *, schema=None) -> None:
+        tmp_transport: asyncio.Transport
 
         pkt_receiver = (
             self.msg_transport.get_extra_info(self.msg_transport.READER)
             if self.msg_transport
             else None
         )
-        _, tmp_transport = create_pkt_stack(self, pkt_receiver, packet_dict=packets)
+        _, tmp_transport = self._create_pkt_stack(pkt_receiver, packet_dict=packets)
         await tmp_transport.get_extra_info(SZ_POLLER_TASK)
 
         # self.msg_transport._clear_write_buffer()  # TODO: shouldn't be needed
         self.msg_protocol._prev_msg = None  # TODO: move to pause/resume?
 
     def get_device(
-        self, dev_id, *, msg=None, parent=None, child_id=None, is_sensor=None
+        self,
+        dev_id: _DeviceIdT,
+        *,
+        msg: None | Message = None,
+        parent=None,
+        child_id=None,
+        is_sensor: None | bool = None,
     ) -> Device:  # TODO: **schema) -> Device:
         """Return a device, create it if required.
 
@@ -500,7 +529,7 @@ class Gateway(Engine):
         If a device is created, attach it to the gateway.
         """
 
-        def check_filter_lists(dev_id: str) -> None:
+        def check_filter_lists(dev_id: _DeviceIdT) -> None:
             """Raise an LookupError if a device_id is filtered out by a list."""
 
             if dev_id in self._unwanted:
@@ -525,7 +554,7 @@ class Gateway(Engine):
                 )
 
         check_filter_lists(dev_id)
-        traits = SCHEMA_DEV(self._include.get(dev_id, {}))
+        traits = SCH_TRAITS(self._include.get(dev_id, {}))
 
         dev = self.device_by_id.get(dev_id)
         if not dev:
@@ -540,13 +569,23 @@ class Gateway(Engine):
             dev.set_parent(parent, child_id=child_id, is_sensor=is_sensor)
 
         if traits.get(SZ_FAKED):
-            dev._make_fake()
+            if isinstance(dev, Fakeable):
+                dev._make_fake()
+            else:
+                _LOGGER.warning(f"The device is not fakable: {dev}")
 
         if msg:
             dev._handle_msg(msg)
 
         return dev
 
+    @property
+    def hgi(self) -> Optional[Device]:
+        """Return the HGI80-compatible gateway device."""
+        if self.pkt_protocol and self.pkt_protocol._hgi80[SZ_DEVICE_ID]:
+            return self.device_by_id.get(self.pkt_protocol._hgi80[SZ_DEVICE_ID])
+        return None
+
     @property
     def tcs(self) -> Optional[System]:
         """Return the primary TCS, if any."""
@@ -576,11 +615,15 @@ class Gateway(Engine):
 
     @property
     def system_by_id(self) -> dict:
-        return {d.id: d.tcs for d in self.devices if getattr(d, "tcs", None)}
+        return {
+            d.id: d.tcs
+            for d in self.devices
+            if hasattr(d, "tcs") and getattr(d.tcs, "id", None) == d.id
+        }  # why something so simple look so messy
 
     @property
     def systems(self) -> list:
-        return [d.tcs for d in self.devices if getattr(d, "tcs", None)]
+        return list(self.system_by_id.values())
 
     @property
     def _config(self) -> dict:
@@ -596,7 +639,7 @@ class Gateway(Engine):
         return {
             "_gateway_id": self.hgi.id if self.hgi else None,
             SZ_MAIN_CONTROLLER: self.tcs.id if self.tcs else None,
-            SZ_CONFIG: {ENFORCE_KNOWN_LIST: self.config.enforce_known_list},
+            SZ_CONFIG: {SZ_ENFORCE_KNOWN_LIST: self.config.enforce_known_list},
             SZ_KNOWN_LIST: self.known_list,
             SZ_BLOCK_LIST: [{k: v} for k, v in self._exclude.items()],
             "_unwanted": sorted(self.pkt_protocol._unwanted),
@@ -641,7 +684,9 @@ class Gateway(Engine):
     def status(self) -> dict:
         return {SZ_DEVICES: {d.id: d.status for d in sorted(self.devices)}}
 
-    def send_cmd(self, cmd: Command, callback: Callable = None, **kwargs) -> Future:
+    def send_cmd(
+        self, cmd: Command, callback: Callable = None, **kwargs
+    ) -> futures.Future:
         """Send a command with the option to return any response via callback."""
 
         if self.config.disable_sending:
@@ -654,7 +699,10 @@ class Gateway(Engine):
         return fut
 
     def fake_device(
-        self, device_id, create_device=False, start_binding=False
+        self,
+        device_id: _DeviceIdT,
+        create_device: bool = False,
+        start_binding: bool = False,
     ) -> Device:
         """Create a faked device, and optionally set it to binding mode.
 
@@ -675,7 +723,9 @@ class Gateway(Engine):
         elif device_id in self._exclude:
             del self._exclude[device_id]
 
-        return self.get_device(device_id)._make_fake(bind=start_binding)
+        if (dev := self.get_device(device_id)) and isinstance(dev, Fakeable):
+            return dev._make_fake(bind=start_binding)
+        raise TypeError(f"The device is not fakable: {device_id}")
 
     def add_task(self, fnc, *args, delay=None, period=None, **kwargs) -> None:
         """Start a task after delay seconds and then repeat it every period seconds."""
@@ -686,18 +736,20 @@ class Gateway(Engine):
 
 
 class GatewayWithSql(Gateway):
+    """The gateway class (with MySQL)."""
+
     def __init__(self, *args, **kwargs) -> None:
 
         import sqlite3
 
-        self._db = sqlite3.connect("file::memory:?cache=shared")
+        self._db: sqlite3.Connection = sqlite3.connect("file::memory:?cache=shared")
         self._cur = self._db.cursor()
 
     async def start(self, *, start_discovery=True) -> None:
-        super().start(start_discovery=start_discovery)
+        await super().start(start_discovery=start_discovery)
 
     async def stop(self) -> None:
-        super().stop()
+        await super().stop()
 
     def _get_state(self, include_expired=None) -> tuple[dict, dict]:
         pass
