@@ -2,10 +2,13 @@
 # -*- coding: utf-8 -*-
 #
 """RAMSES RF - a RAMSES-II protocol decoder & analyser."""
+from __future__ import annotations
 
 import re
 from types import SimpleNamespace
 
+from .backports import StrEnum
+
 __dev_mode__ = False
 DEV_MODE = __dev_mode__
 
@@ -44,6 +47,8 @@ SZ_EXHAUST_FAN_SPEED = "exhaust_fan_speed"
 SZ_EXHAUST_FLOW = "exhaust_flow"
 SZ_EXHAUST_TEMPERATURE = "exhaust_temperature"
 SZ_FAN_INFO = "fan_info"
+SZ_FAN_MODE = "fan_mode"
+SZ_FAN_RATE = "fan_rate"
 SZ_INDOOR_HUMIDITY = "indoor_humidity"
 SZ_INDOOR_TEMPERATURE = "indoor_temperature"
 SZ_OUTDOOR_HUMIDITY = "outdoor_humidity"
@@ -73,10 +78,10 @@ class AttrDict(dict):
     __delitem__ = __readonly
     __setitem__ = __readonly
     clear = __readonly
-    pop = __readonly
-    popitem = __readonly
-    setdefault = __readonly
-    update = __readonly
+    pop = __readonly  # type:ignore[assignment]
+    popitem = __readonly  # type:ignore[assignment]
+    setdefault = __readonly  # type:ignore[assignment]
+    update = __readonly  # type:ignore[assignment]
 
     del __readonly
 
@@ -245,6 +250,7 @@ DEV_ROLE_MAP = attr_dict_factory(
     },
 )
 
+
 # slugs for device entity types, used in device_ids
 DEV_TYPE = SimpleNamespace(
     #
@@ -282,7 +288,7 @@ DEV_TYPE = SimpleNamespace(
     CO2="CO2",  # ??: HVAC CO2 sensor
     HUM="HUM",  # ??: HVAC humidity sensor, 1260: 32
     PIR="PIR",  # ??: HVAC pesence sensor, 2E10
-    SWI="SWI",  # ??: HVAC switch, 22F[13]: 02|06|20|32|39|42|49|59 (no 20: are both)
+    REM="REM",  # ??: HVAC switch, 22F[13]: 02|06|20|32|39|42|49|59 (no 20: are both)
     SW2="SW2",  # ??: HVAC switch, Orcon variant
     DIS="DIS",  # ??: HVAC switch with display
 )
@@ -320,7 +326,7 @@ DEV_TYPE_MAP = attr_dict_factory(
         DEV_TYPE.HUM: {None: "rh_sensor"},
         DEV_TYPE.PIR: {None: "presence_sensor"},
         DEV_TYPE.RFS: {None: "hvac_gateway"},  # Spider
-        DEV_TYPE.SWI: {None: "switch"},
+        DEV_TYPE.REM: {None: "switch"},
         DEV_TYPE.SW2: {None: "switch_variant"},
     },
     {
@@ -350,7 +356,7 @@ DEV_TYPE_MAP = attr_dict_factory(
             DEV_TYPE.FAN: "ventilator",  # Both Fans and HRUs
             DEV_TYPE.HUM: "rh_sensor",
             DEV_TYPE.RFS: "hvac_gateway",  # Spider
-            DEV_TYPE.SWI: "switch",
+            DEV_TYPE.REM: "switch",
         },
     },
 )
@@ -450,11 +456,6 @@ SZ_ZONE_MASK = "zone_mask"
 SZ_ZONE_TYPE = "zone_type"
 SZ_ZONES = "zones"
 
-SZ_ALIAS = "alias"
-SZ_CLASS = "class"
-SZ_INBOUND = "inbound"
-SZ_OUTBOUND = "outbound"
-
 
 DEFAULT_MAX_ZONES = 16 if DEV_MODE else 12
 # Evohome: 12 (0-11), older/initial version was 8
@@ -474,9 +475,10 @@ DEVICE_ID_REGEX = SimpleNamespace(
 )
 
 # Domains
-F8, F9, FA, FB, FC, FD, FE, FF = (f"{x:02X}" for x in range(0xF8, 0x100))
+F6, F7, F8, F9, FA, FB, FC, FD, FE, FF = (f"{x:02X}" for x in range(0xF6, 0x100))
 
 DOMAIN_TYPE_MAP = {
+    F6: "cooling_valve",  # cooling
     F8: None,
     F9: DEV_ROLE_MAP[DEV_ROLE.HT1],  # Heating Valve
     FA: DEV_ROLE_MAP[DEV_ROLE.HTG],  # HW Valve (or UFH loop if src.type == UFC?)
@@ -497,11 +499,11 @@ DTM_TIME_REGEX = re.compile(r"[0-2]\d:[0-5]\d:[0-5]\d\.\d{3} ?")  # 13:15:00.123
 
 # Used by packet structure validators
 r = r"(-{3}|\d{3}|\.{3})"  # RSSI, '...' was used by an older version of evofw3
-v = r"( I|RP|RQ| W)"  # Verb
-d = r"(-{2}:-{6}|\d{2}:\d{6})"  # Device ID
-c = r"[0-9A-F]{4}"  # Code
-l = r"\d{3}"  # Length # noqa: E741
-p = r"([0-9A-F]{2}){1,48}"  # Payload
+v = r"( I|RP|RQ| W)"  # verb
+d = r"(-{2}:-{6}|\d{2}:\d{6})"  # device ID
+c = r"[0-9A-F]{4}"  # code
+l = r"\d{3}"  # length # noqa: E741
+p = r"([0-9A-F]{2}){1,48}"  # payload
 
 # DEVICE_ID_REGEX = re.compile(f"^{d}$")
 COMMAND_REGEX = re.compile(f"^{v} {r} {d} {d} {d} {c} {l} {p}$")
@@ -543,97 +545,6 @@ SystemType = SimpleNamespace(
 )
 
 
-I_, RQ, RP, W_ = " I", "RQ", "RP", " W"
-
-_0001 = "0001"
-_0002 = "0002"
-_0004 = "0004"
-_0005 = "0005"
-_0006 = "0006"
-_0008 = "0008"
-_0009 = "0009"
-_000A = "000A"
-_000C = "000C"
-_000E = "000E"
-_0016 = "0016"
-_0100 = "0100"
-_0150 = "0150"
-_01D0 = "01D0"
-_01E9 = "01E9"
-_0404 = "0404"
-_0418 = "0418"
-_042F = "042F"
-_0B04 = "0B04"
-_1030 = "1030"
-_1060 = "1060"
-_1081 = "1081"
-_1090 = "1090"
-_1098 = "1098"
-_10A0 = "10A0"
-_10B0 = "10B0"
-_10E0 = "10E0"
-_10E1 = "10E1"
-_10E2 = "10E2"
-_1100 = "1100"
-_11F0 = "11F0"
-_1260 = "1260"
-_1280 = "1280"
-_1290 = "1290"
-_1298 = "1298"
-_12A0 = "12A0"
-_12B0 = "12B0"
-_12C0 = "12C0"
-_12C8 = "12C8"
-_12F0 = "12F0"
-_1300 = "1300"
-_1470 = "1470"
-_1F09 = "1F09"
-_1F70 = "1F70"
-_1F41 = "1F41"
-_1FC9 = "1FC9"
-_1FCA = "1FCA"
-_1FD0 = "1FD0"
-_1FD4 = "1FD4"
-_2249 = "2249"
-_22C9 = "22C9"
-_22D0 = "22D0"
-_22D9 = "22D9"
-_22F1 = "22F1"
-_22F3 = "22F3"
-_22F7 = "22F7"
-_22F8 = "22F8"
-_22B0 = "22B0"
-_2309 = "2309"
-_2349 = "2349"
-_2389 = "2389"
-_2400 = "2400"
-_2401 = "2401"
-_2410 = "2410"
-_2411 = "2411"
-_2420 = "2420"
-_2D49 = "2D49"
-_2E04 = "2E04"
-_2E10 = "2E10"
-_30C9 = "30C9"
-_3110 = "3110"
-_3120 = "3120"
-_313F = "313F"
-_3150 = "3150"
-_31D9 = "31D9"
-_31DA = "31DA"
-_31E0 = "31E0"
-_3200 = "3200"
-_3210 = "3210"
-_3220 = "3220"
-_3221 = "3221"
-_3223 = "3223"
-_3B00 = "3B00"
-_3EF0 = "3EF0"
-_3EF1 = "3EF1"
-_4401 = "4401"
-_PUZZ = "7FFF"
-
-
 # used by 22Fx parser, and FanSwitch devices
 # SZ_BOOST_TIMER = "boost_timer"  # minutes, e.g. 10, 20, 30 minutes
 HEATER_MODE = "heater_mode"  # e.g. auto, off
@@ -642,14 +553,14 @@ FAN_RATE = "fan_rate"  # percentage, 0.0 - 1.0
 
 
 # RP --- 01:054173 18:006402 --:------ 0005 004 00100000  # before adding RFG100
-#  I --- 01:054173 --:------ 01:054173 1FC9 012 0010E004D39D001FC904D39D
-#  W --- 30:248208 01:054173 --:------ 1FC9 012 0010E07BC9900012907BC990
-#  I --- 01:054173 30:248208 --:------ 1FC9 006 00FFFF04D39D
+# .I --- 01:054173 --:------ 01:054173 1FC9 012 0010E004D39D001FC904D39D
+# .W --- 30:248208 01:054173 --:------ 1FC9 012 0010E07BC9900012907BC990
+# .I --- 01:054173 30:248208 --:------ 1FC9 006 00FFFF04D39D
 
 # RP --- 01:054173 18:006402 --:------ 0005 004 00100100  # after adding RFG100
 # RP --- 01:054173 18:006402 --:------ 000C 006 0010007BC990  # 30:082155
 # RP --- 01:054173 18:006402 --:------ 0005 004 00100100  # before deleting RFG from CTL
-#  I --- 01:054173 --:------ 01:054173 0005 004 00100000  # when the RFG was deleted
+# .I --- 01:054173 --:------ 01:054173 0005 004 00100000  # when the RFG was deleted
 # RP --- 01:054173 18:006402 --:------ 0005 004 00100000  # after deleting the RFG
 
 # RP|zone_devices | 000E0... || {'domain_id': 'FA', 'device_role': 'dhw_valve', 'devices': ['13:081807']}  # noqa: E501
@@ -660,3 +571,108 @@ FAN_RATE = "fan_rate"  # percentage, 0.0 - 1.0
 
 # HCW80 has option of being wired (normally wireless)
 # ST9420C has battery back-up (as does evohome)
+
+
+I_ = " I"
+RQ = "RQ"
+RP = "RP"
+W_ = " W"
+
+
+class Code(StrEnum):
+    _0001 = "0001"
+    _0002 = "0002"
+    _0004 = "0004"
+    _0005 = "0005"
+    _0006 = "0006"
+    _0008 = "0008"
+    _0009 = "0009"
+    _000A = "000A"
+    _000C = "000C"
+    _000E = "000E"
+    _0016 = "0016"
+    _0100 = "0100"
+    _0150 = "0150"
+    _01D0 = "01D0"
+    _01E9 = "01E9"
+    _0404 = "0404"
+    _0418 = "0418"
+    _042F = "042F"
+    _0B04 = "0B04"
+    _1030 = "1030"
+    _1060 = "1060"
+    _1081 = "1081"
+    _1090 = "1090"
+    _1098 = "1098"
+    _10A0 = "10A0"
+    _10B0 = "10B0"
+    _10D0 = "10D0"  # Orcon
+    _10E0 = "10E0"
+    _10E1 = "10E1"
+    _10E2 = "10E2"
+    _1100 = "1100"
+    _11F0 = "11F0"
+    _1260 = "1260"
+    _1280 = "1280"
+    _1290 = "1290"
+    _1298 = "1298"
+    _12A0 = "12A0"
+    _12B0 = "12B0"
+    _12C0 = "12C0"
+    _12C8 = "12C8"
+    _12F0 = "12F0"
+    _1300 = "1300"
+    _1470 = "1470"  # Orcon
+    _1F09 = "1F09"
+    _1F41 = "1F41"
+    _1F70 = "1F70"
+    _1FC9 = "1FC9"
+    _1FCA = "1FCA"
+    _1FD0 = "1FD0"
+    _1FD4 = "1FD4"
+    _2210 = "2210"  # Orcon
+    _2249 = "2249"
+    _22C9 = "22C9"
+    _22D0 = "22D0"
+    _22D9 = "22D9"
+    _22E0 = "22E0"  # Orcon
+    _22E5 = "22E5"  # Orcon
+    _22E9 = "22E9"  # Orcon
+    _22F1 = "22F1"
+    _22F2 = "22F2"  # Orcon
+    _22F3 = "22F3"
+    _22F4 = "22F4"  # Orcon
+    _22F7 = "22F7"  # Orcon
+    _22F8 = "22F8"  # Orcon
+    _22B0 = "22B0"
+    _2309 = "2309"
+    _2349 = "2349"
+    _2389 = "2389"
+    _2400 = "2400"
+    _2401 = "2401"
+    _2410 = "2410"
+    _2411 = "2411"
+    _2420 = "2420"
+    _2D49 = "2D49"
+    _2E04 = "2E04"
+    _2E10 = "2E10"
+    _30C9 = "30C9"
+    _3110 = "3110"
+    _3120 = "3120"
+    _313E = "313E"  # Orcon
+    _313F = "313F"
+    _3150 = "3150"
+    _31D9 = "31D9"
+    _31DA = "31DA"
+    _31E0 = "31E0"
+    _3200 = "3200"
+    _3210 = "3210"
+    _3220 = "3220"
+    _3221 = "3221"
+    _3222 = "3222"  # Orcon
+    _3223 = "3223"
+    _3B00 = "3B00"
+    _3EF0 = "3EF0"
+    _3EF1 = "3EF1"
+    _4401 = "4401"
+    _PUZZ = "7FFF"
