@@ -5,11 +5,9 @@
 
 Provide the base class for commands (constructed/sent packets) and packets.
 """
-
-# TODO: add _has_idx (ass func return only one type, or raise)
+from __future__ import annotations
 
 import logging
-from typing import Optional, Union
 
 from .address import NON_DEV_ADDR, NUL_DEV_ADDR, Address, pkt_addrs
 from .const import COMMAND_REGEX, DEV_ROLE_MAP, DEV_TYPE_MAP, __dev_mode__
@@ -25,6 +23,9 @@ from .ramses import (
     RQ_NO_PAYLOAD,
 )
 
+# TODO: add _has_idx (ass func return only one type, or raise)
+
+
 # skipcq: PY-W2000
 from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     I_,
@@ -36,90 +37,9 @@ from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     FA,
     FC,
     FF,
+    Code,
 )
 
-# skipcq: PY-W2000
-from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
-    _0001,
-    _0002,
-    _0004,
-    _0005,
-    _0006,
-    _0008,
-    _0009,
-    _000A,
-    _000C,
-    _000E,
-    _0016,
-    _0100,
-    _0150,
-    _01D0,
-    _01E9,
-    _0404,
-    _0418,
-    _042F,
-    _0B04,
-    _1030,
-    _1060,
-    _1081,
-    _1090,
-    _1098,
-    _10A0,
-    _10B0,
-    _10E0,
-    _10E1,
-    _1100,
-    _11F0,
-    _1260,
-    _1280,
-    _1290,
-    _1298,
-    _12A0,
-    _12B0,
-    _12C0,
-    _12C8,
-    _12F0,
-    _1300,
-    _1F09,
-    _1F41,
-    _1FC9,
-    _1FCA,
-    _1FD0,
-    _1FD4,
-    _2249,
-    _22C9,
-    _22D0,
-    _22D9,
-    _22F1,
-    _22F3,
-    _2309,
-    _2349,
-    _2389,
-    _2400,
-    _2401,
-    _2410,
-    _2420,
-    _2D49,
-    _2E04,
-    _2E10,
-    _30C9,
-    _3110,
-    _3120,
-    _313F,
-    _3150,
-    _31D9,
-    _31DA,
-    _31E0,
-    _3200,
-    _3210,
-    _3220,
-    _3221,
-    _3223,
-    _3B00,
-    _3EF0,
-    _3EF1,
-    _PUZZ,
-)
 
 DEV_MODE = __dev_mode__ and False
 
@@ -128,6 +48,13 @@ if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
+_CodeT = str
+_DeviceIdT = str
+_HeaderT = str
+_PayloadT = str
+_VerbT = str
+
+
 class Frame:
     """The Frame class - used as a base by the Command and Packet classes.
 
@@ -136,15 +63,15 @@ class Frame:
 
     _frame: str
 
-    verb: str
+    verb: _VerbT
     seqn: str  # TODO: or, better as int?
     src: Address
     dst: Address
     _addrs: tuple[Address, Address, Address]
-    code: str
+    code: _CodeT
     len_: str
     _len: int  # int(len(payload) / 2)
-    payload: str
+    payload: _PayloadT
 
     def __init__(self, frame: str) -> None:
         """Create a frame from a string.
@@ -168,7 +95,7 @@ class Frame:
         self._len = int(len(self.payload) / 2)
 
         try:
-            self.src, self.dst, *self._addrs = pkt_addrs(
+            self.src, self.dst, *self._addrs = pkt_addrs(  # type: ignore[assignment]
                 " ".join(fields[i] for i in range(2, 5))  # frame[7:36]
             )
         except InvalidPacketError as exc:  # will be: InvalidAddrSetError
@@ -180,23 +107,25 @@ class Frame:
                 f"len({self.payload}) is not int('{self.len_}' * 2))"
             )
 
-        self._ctx_: Union[str, bool] = None  # type: ignore[assignment]
+        self._ctx_: bool | str = None  # type: ignore[assignment]
         self._hdr_: str = None  # type: ignore[assignment]
-        self._idx_: Union[str, bool] = None  # type: ignore[assignment]
+        self._idx_: bool | str = None  # type: ignore[assignment]
 
         self._has_array_: bool = None  # type: ignore[assignment]
         self._has_ctl_: bool = None  # type: ignore[assignment]  # TODO: remove
         self._has_payload_: bool = None  # type: ignore[assignment]
 
     @classmethod  # for internal use only
-    def _from_attrs(cls, verb, *addrs, code, payload, seqn=None):
+    def _from_attrs(
+        cls, verb: _VerbT, *addrs, code: _CodeT, payload: _PayloadT, seqn=None
+    ):
         """Create a frame from its attributes (args, kwargs)."""
 
         seqn = seqn or "---"
         len_ = f"{int(len(payload) / 2):03d}"
 
         try:
-            return cls(" ".join(verb, seqn, *addrs, code, len_, payload))
+            return cls(" ".join((verb, seqn, *addrs, code, len_, payload)))
         except TypeError as exc:
             raise InvalidPacketError(f"Bad frame: invalid attr: {exc}")
 
@@ -216,7 +145,7 @@ class Frame:
             raise InvalidPacketError("Bad frame: payload length mismatch")
 
         try:
-            self.src, self.dst, *self._addrs = pkt_addrs(self._frame[7:36])
+            self.src, self.dst, *self._addrs = pkt_addrs(self._frame[7:36])  # type: ignore[assignment]
         except InvalidPacketError as exc:  # will be: InvalidAddrSetError
             raise InvalidPacketError(f"Bad frame: invalid address set: {exc}")
 
@@ -251,7 +180,7 @@ class Frame:
             return f"{self!r} < {exc}"
 
     @property
-    def _has_array(self) -> Optional[bool]:
+    def _has_array(self) -> None | bool:
         """Return the True if the packet payload is an array, False if not.
 
         May return false negatives (e.g. arrays of length 1), and None if undetermined.
@@ -265,11 +194,11 @@ class Frame:
 
         # False -ves (array length is 1) are an acceptable compromise to extensive checking
 
-        #  W --- 01:145038 34:092243 --:------ 1FC9 006 07230906368E
-        #  I --- 01:145038 --:------ 01:145038 1FC9 018 07000806368E-FC3B0006368E-071FC906368E
-        #  I --- 01:145038 --:------ 01:145038 1FC9 018 FA000806368E-FC3B0006368E-FA1FC906368E
-        #  I --- 34:092243 --:------ 34:092243 1FC9 030 0030C9896853-002309896853-001060896853-0010E0896853-001FC9896853
-        if self.code == _1FC9:
+        # .W --- 01:145038 34:092243 --:------ 1FC9 006 07230906368E
+        # .I --- 01:145038 --:------ 01:145038 1FC9 018 07000806368E-FC3B0006368E-071FC906368E
+        # .I --- 01:145038 --:------ 01:145038 1FC9 018 FA000806368E-FC3B0006368E-FA1FC906368E
+        # .I --- 34:092243 --:------ 34:092243 1FC9 030 0030C9896853-002309896853-001060896853-0010E0896853-001FC9896853
+        if self.code == Code._1FC9:
             self._has_array_ = self.verb != RQ  # safe to treat all as array, even len=1
 
         elif self.verb != I_ or self.code not in CODES_WITH_ARRAYS:
@@ -278,7 +207,7 @@ class Frame:
         elif self._len == CODES_WITH_ARRAYS[self.code][0]:  # NOTE: can be false -ves
             self._has_array_ = False
             if (
-                self.code in (_22C9, _3150)  # only time 22C9 is seen
+                self.code in (Code._22C9, Code._3150)  # only time 22C9 is seen
                 and self.src.type == DEV_TYPE_MAP.UFC
                 and self.dst is self.src
                 and self.payload[:1] != "F"
@@ -300,21 +229,21 @@ class Frame:
             ), f"{self} < array is from a non-controller (02)"
             self._has_array_ = True
 
-        #  I --- 10:040239 01:223036 --:------ 0009 003 000000        # not array
-        #  I --- 01:102458 --:------ 01:102458 0009 006 FC01FF-F901FF
-        #  I --- 01:145038 --:------ 01:145038 0009 006 FC00FF-F900FF
-        #  I 034 --:------ --:------ 12:126457 2309 006 017EFF-027EFF
-        #  I --- 01:223036 --:------ 01:223036 000A 012 081001F40DAC-091001F40DAC  # 2nd fragment
-        #  I 024 --:------ --:------ 12:126457 000A 012 010001F40BB8-020001F40BB8
-        #  I --- 02:044328 --:------ 02:044328 22C9 018 0001F40A2801-0101F40A2801-0201F40A2801
-        #  I --- 23:100224 --:------ 23:100224 2249 007 007EFF7EFFFFFF  # can have 2 zones
-        #  I --- 02:044328 --:------ 02:044328 22C9 018 0001F40A2801-0101F40A2801-0201F40A2801
-        #  I --- 02:001107 --:------ 02:001107 3150 010 007A-017A-027A-036A-046A
+        # .I --- 10:040239 01:223036 --:------ 0009 003 000000        # not array
+        # .I --- 01:102458 --:------ 01:102458 0009 006 FC01FF-F901FF
+        # .I --- 01:145038 --:------ 01:145038 0009 006 FC00FF-F900FF
+        # .I 034 --:------ --:------ 12:126457 2309 006 017EFF-027EFF
+        # .I --- 01:223036 --:------ 01:223036 000A 012 081001F40DAC-091001F40DAC  # 2nd fragment
+        # .I 024 --:------ --:------ 12:126457 000A 012 010001F40BB8-020001F40BB8
+        # .I --- 02:044328 --:------ 02:044328 22C9 018 0001F40A2801-0101F40A2801-0201F40A2801
+        # .I --- 23:100224 --:------ 23:100224 2249 007 007EFF7EFFFFFF  # can have 2 zones
+        # .I --- 02:044328 --:------ 02:044328 22C9 018 0001F40A2801-0101F40A2801-0201F40A2801
+        # .I --- 02:001107 --:------ 02:001107 3150 010 007A-017A-027A-036A-046A
 
         return self._has_array_
 
     @property
-    def _has_ctl(self) -> Optional[bool]:
+    def _has_ctl(self) -> None | bool:
         """Return True if the packet is to/from a controller."""
 
         if self._has_ctl_ is not None:
@@ -330,29 +259,33 @@ class Frame:
             _LOGGER.debug(f"{self} # HAS controller (10)")
             self._has_ctl_ = True
 
-        #  I --- 12:010740 --:------ 12:010740 30C9 003 0008D9 # not ctl
+        # .I --- 12:010740 --:------ 12:010740 30C9 003 0008D9 # not ctl
         elif self.dst is self.src:  # (not needed?) & self.code == I_:
             _LOGGER.debug(
                 f"{self} < "
-                + ("HAS" if self.code in CODES_ONLY_FROM_CTL + [_31D9, _31DA] else "no")
+                + (
+                    "HAS"
+                    if self.code in CODES_ONLY_FROM_CTL + (Code._31D9, Code._31DA)
+                    else "no"
+                )
                 + " controller (20)"
             )
             self._has_ctl_ = any(
                 (
-                    self.code == _3B00 and self.payload[:2] == FC,
-                    self.code in CODES_ONLY_FROM_CTL + [_31D9, _31DA],
+                    self.code == Code._3B00 and self.payload[:2] == FC,
+                    self.code in CODES_ONLY_FROM_CTL + (Code._31D9, Code._31DA),
                 )
             )
 
-        #  I --- --:------ --:------ 10:050360 1FD4 003 002ABE # no ctl
-        #  I 095 --:------ --:------ 12:126457 1F09 003 000BC2 # HAS ctl
-        #  I --- --:------ --:------ 20:001473 31D9 003 000001 # ctl? (HVAC)
+        # .I --- --:------ --:------ 10:050360 1FD4 003 002ABE # no ctl
+        # .I 095 --:------ --:------ 12:126457 1F09 003 000BC2 # HAS ctl
+        # .I --- --:------ --:------ 20:001473 31D9 003 000001 # ctl? (HVAC)
         elif self.dst.id == NON_DEV_ADDR.id:
             _LOGGER.debug(f"{self} # HAS controller (21)")
             self._has_ctl_ = self.src.type != DEV_TYPE_MAP.OTB  # DEX
 
-        #  I --- 10:037879 --:------ 12:228610 3150 002 0000   # HAS ctl
-        #  I --- 04:029390 --:------ 12:126457 1060 003 01FF01 # HAS ctl
+        # .I --- 10:037879 --:------ 12:228610 3150 002 0000   # HAS ctl
+        # .I --- 04:029390 --:------ 12:126457 1060 003 01FF01 # HAS ctl
         elif self.dst.type in (DEV_TYPE_MAP.DTS, DEV_TYPE_MAP.DT2):  # DEX
             _LOGGER.debug(f"{self} # HAS controller (22)")
             self._has_ctl_ = True
@@ -369,8 +302,8 @@ class Frame:
         # RQ --- 30:258720 10:050360 --:------ 3EF0 001 00
         # RP --- 10:050360 30:258720 --:------ 3EF0 006 000011010A1C
 
-        #  I --- 34:021943 63:262142 --:------ 10E0 038 000001C8380A01... # unknown
-        #  I --- 32:168090 30:082155 --:------ 31E0 004 0000C800          # unknown
+        # .I --- 34:021943 63:262142 --:------ 10E0 038 000001C8380A01... # unknown
+        # .I --- 32:168090 30:082155 --:------ 31E0 004 0000C800          # unknown
         if self._has_ctl_ is None:
             if DEV_MODE and DEV_TYPE_MAP.HGI not in (
                 self.src.type,
@@ -395,9 +328,9 @@ class Frame:
             (
                 self._len == 1,
                 self.verb == RQ and self.code in RQ_NO_PAYLOAD,
-                self.verb == RQ and self._len == 2 and self.code != _0016,
+                self.verb == RQ and self._len == 2 and self.code != Code._0016,
                 # self.verb == RQ and self._len == 2 and self.code in (
-                #   _2309, _2349, _3EF1
+                #   Code._2309, Code._2349, Code._3EF1
                 # ),
             )
         )
@@ -411,23 +344,26 @@ class Frame:
         self._idx_ = None  # type: ignore[assignment]
 
     @property
-    def _ctx(self) -> Union[str, bool]:  # incl. self._idx
+    def _ctx(self) -> bool | str:  # incl. self._idx
         """Return the payload's full context, if any (e.g. for 0404: zone_idx/frag_idx).
 
         Used to store packets in the entity's message DB. It is a superset of _idx.
         """
 
         if self._ctx_ is None:
-            if self.code in (_0005, _000C):  # zone_idx, zone_type (device_role)
+            if self.code in (
+                Code._0005,
+                Code._000C,
+            ):  # zone_idx, zone_type (device_role)
                 self._ctx_ = self.payload[:4]
-            elif self.code == _0404:  # zone_idx, frag_idx
+            elif self.code == Code._0404:  # zone_idx, frag_idx
                 self._ctx_ = self._idx + self.payload[10:12]
             else:
                 self._ctx_ = self._idx
         return self._ctx_
 
     @property
-    def _hdr(self) -> str:  # incl. self._ctx
+    def _hdr(self) -> _HeaderT:  # incl. self._ctx
         """Return the QoS header (fingerprint) of this packet (i.e. device_id/code/hdr).
 
         Used for QoS (timeouts, retries), callbacks, etc.
@@ -439,7 +375,7 @@ class Frame:
         return self._hdr_
 
     @property
-    def _idx(self) -> Union[str, bool]:
+    def _idx(self) -> bool | str:
         """Return the payload's index, if any (e.g. zone_idx, domain_id  or log_idx).
 
         Used to route a packet to the correct entity's (i.e. zone/domain) msg handler.
@@ -450,7 +386,7 @@ class Frame:
         return self._idx_
 
 
-def _pkt_idx(pkt) -> Union[str, bool, None]:  # _has_array, _has_ctl
+def _pkt_idx(pkt) -> None | bool | str:  # _has_array, _has_ctl
     """Return the payload's 2-byte context (e.g. zone_idx, domain_id or log_idx).
 
     May return a 2-byte string (usu. pkt.payload[:2]), or:
@@ -463,14 +399,14 @@ def _pkt_idx(pkt) -> Union[str, bool, None]:  # _has_array, _has_ctl
     # FIXME: 0016 is broken
 
     # mutex 2/4, CODE_IDX_COMPLEX: are not payload[:2]
-    if pkt.code == _0005:
+    if pkt.code == Code._0005:
         return pkt._has_array
 
-    #  I --- 10:040239 01:223036 --:------ 0009 003 000000
-    if pkt.code == _0009 and pkt.src.type == DEV_TYPE_MAP.OTB:  # DEX
+    # .I --- 10:040239 01:223036 --:------ 0009 003 000000
+    if pkt.code == Code._0009 and pkt.src.type == DEV_TYPE_MAP.OTB:  # DEX
         return False
 
-    if pkt.code == _000C:  # zone_idx/domain_id (complex, payload[0:4])
+    if pkt.code == Code._000C:  # zone_idx/domain_id (complex, payload[0:4])
         if pkt.payload[2:4] == DEV_ROLE_MAP.APP:  # "000F"
             return FC
         if pkt.payload[0:4] == f"01{DEV_ROLE_MAP.HTG}":  # "010E"
@@ -482,16 +418,16 @@ def _pkt_idx(pkt) -> Union[str, bool, None]:  # _has_array, _has_ctl
             return FA
         return pkt.payload[:2]
 
-    if pkt.code == _0404:
-        return FA if pkt.payload[2:4] == "23" else pkt.payload[:2]
+    if pkt.code == Code._0404:  # assumes only 1 DHW zone (can be 2, but never seen)
+        return "HW" if pkt.payload[2:4] == "23" else pkt.payload[:2]
 
-    if pkt.code == _0418:  # log_idx (payload[4:6])
+    if pkt.code == Code._0418:  # log_idx (payload[4:6])
         return pkt.payload[4:6]
 
-    if pkt.code == _1100:  # TODO; can do in parser
+    if pkt.code == Code._1100:  # TODO; can do in parser
         return pkt.payload[:2] if pkt.payload[:1] == "F" else False  # only FC
 
-    if pkt.code == _3220:  # msg_id/data_id (payload[4:6])
+    if pkt.code == Code._3220:  # msg_id/data_id (payload[4:6])
         return pkt.payload[4:6]
 
     if pkt.code in CODE_IDX_COMPLEX:
@@ -523,10 +459,10 @@ def _pkt_idx(pkt) -> Union[str, bool, None]:  # _has_array, _has_ctl
         pkt._has_ctl
     ):  # risk of false -ves, TODO: pkt.src.type == DEV_TYPE_MAP.HGI too?  # DEX
         # 02:    22C9: would be picked up as an array, if len==1 counted
-        # 03:    #  I 028 03:094242 --:------ 03:094242 30C9 003 010B22  # ctl
+        # 03:    # .I 028 03:094242 --:------ 03:094242 30C9 003 010B22  # ctl
         # 12/22: 000A|1030|2309|30C9 from (addr0 --:), 1060|3150 (addr0 04:)
         # 23:    0009|10A0
-        return pkt.payload[:2]  # pkt._gwy.config.max_zones checked elsewhere
+        return pkt.payload[:2]  # tcs._max_zones checked elsewhere
 
     if pkt.payload[:2] != "00":
         raise InvalidPayloadError(
@@ -541,16 +477,18 @@ def _pkt_idx(pkt) -> Union[str, bool, None]:  # _has_array, _has_ctl
     return None
 
 
-def pkt_header(pkt, rx_header=None) -> Optional[str]:  # NOTE: used in command.py
+def pkt_header(
+    pkt, rx_header: bool = None
+) -> None | _HeaderT:  # NOTE: used in command.py
     """Return the QoS header of a packet (all packets have a header).
 
     For rx_header=True, return the header of the response packet, if one is expected.
     """
 
-    if pkt.code == _1FC9:
-        #  I --- 34:021943 --:------ 34:021943 1FC9 024 00-2309-8855B7 00-1FC9-8855B7
-        #  W --- 01:145038 34:021943 --:------ 1FC9 006 00-2309-06368E  # wont know src until it arrives
-        #  I --- 34:021943 01:145038 --:------ 1FC9 006 00-2309-8855B7
+    if pkt.code == Code._1FC9:
+        # .I --- 34:021943 --:------ 34:021943 1FC9 024 00-2309-8855B7 00-1FC9-8855B7
+        # .W --- 01:145038 34:021943 --:------ 1FC9 006 00-2309-06368E  # wont know src until it arrives
+        # .I --- 34:021943 01:145038 --:------ 1FC9 006 00-2309-8855B7
         if not rx_header:
             device_id = NUL_DEV_ADDR.id if pkt.src == pkt.dst else pkt.dst.id
             return "|".join((pkt.code, pkt.verb, device_id))
