@@ -36,6 +36,7 @@ from .const import (
     SZ_EXHAUST_FLOW,
     SZ_EXHAUST_TEMPERATURE,
     SZ_FAN_INFO,
+    SZ_FAN_MODE,
     SZ_FRAG_LENGTH,
     SZ_FRAG_NUMBER,
     SZ_FRAGMENT,
@@ -79,17 +80,17 @@ from .fingerprints import check_signature
 from .helpers import (
     bool_from_hex,
     date_from_hex,
-    double,
+    double_from_hex,
     dtm_from_hex,
     dts_from_hex,
     flag8,
-    percent,
+    percent_from_hex,
     str_from_hex,
     temp_from_hex,
     valve_demand,
 )
 from .opentherm import EN, MSG_DESC, MSG_ID, MSG_NAME, MSG_TYPE, OtMsgType, decode_frame
-from .ramses import _2411_PARAMS_SCHEMA
+from .ramses import _31DA_FAN_INFO, _2411_PARAMS_SCHEMA
 from .version import VERSION
 
 # Kudos & many thanks to:
@@ -222,7 +223,7 @@ def parser_0001(payload, msg) -> dict:
 @parser_decorator  # outdoor_sensor (outdoor_weather / outdoor_temperature)
 def parser_0002(payload, msg) -> dict:
     # seen with: 03:125829, 03:196221, 03:196196, 03:052382, 03:201498, 03:201565:
-    #  I 000 03:201565 --:------ 03:201565 0002 004 03020105  # no zone_idx, domain_id
+    # .I 000 03:201565 --:------ 03:201565 0002 004 03020105  # no zone_idx, domain_id
 
     # is it CODE_IDX_COMPLEX:
     #  - 02...... for outside temp?
@@ -248,9 +249,9 @@ def parser_0004(payload, msg) -> dict:
 
 @parser_decorator  # system_zones (add/del a zone?)
 def parser_0005(payload, msg) -> dict | list[dict]:  # TODO: needs a cleanup
-    #  I --- 01:145038 --:------ 01:145038 0005 004 00000100
+    # .I --- 01:145038 --:------ 01:145038 0005 004 00000100
     # RP --- 02:017205 18:073736 --:------ 0005 004 0009001F
-    #  I --- 34:064023 --:------ 34:064023 0005 012 000A0000-000F0000-00100000
+    # .I --- 34:064023 --:------ 34:064023 0005 012 000A0000-000F0000-00100000
 
     def _parser(seqx) -> dict:
         if msg.src.type == DEV_TYPE_MAP.UFC:  # DEX, or use: seqx[2:4] == ...
@@ -304,10 +305,10 @@ def parser_0008(payload, msg) -> dict:
     # https://www.domoticaforum.eu/viewtopic.php?f=7&t=5806&start=105#p73681
     # e.g. Electric Heat Zone
 
-    #  I --- 01:145038 --:------ 01:145038 0008 002 0314
-    #  I --- 01:145038 --:------ 01:145038 0008 002 F914
-    #  I --- 01:054173 --:------ 01:054173 0008 002 FA00
-    #  I --- 01:145038 --:------ 01:145038 0008 002 FC14
+    # .I --- 01:145038 --:------ 01:145038 0008 002 0314
+    # .I --- 01:145038 --:------ 01:145038 0008 002 F914
+    # .I --- 01:054173 --:------ 01:054173 0008 002 FA00
+    # .I --- 01:145038 --:------ 01:145038 0008 002 FC14
 
     # RP --- 13:109598 18:199952 --:------ 0008 002 0000
     # RP --- 13:109598 18:199952 --:------ 0008 002 00C8
@@ -319,7 +320,7 @@ def parser_0008(payload, msg) -> dict:
             "blob": payload[8:],
         }
 
-    return {SZ_RELAY_DEMAND: percent(payload[2:4])}  # 3EF0[2:4], 3EF1[10:12]
+    return {SZ_RELAY_DEMAND: percent_from_hex(payload[2:4])}  # 3EF0[2:4], 3EF1[10:12]
 
 
 @parser_decorator  # relay_failsafe
@@ -334,11 +335,11 @@ def parser_0009(payload, msg) -> dict | list:
     This setting may need to be enabled to ensure prost protect mode.
     """
     # can get: 003 or 006, e.g.: FC01FF-F901FF or FC00FF-F900FF
-    #  I --- 23:100224 --:------ 23:100224 0009 003 0100FF  # 2-zone ST9520C
-    #  I --- 10:040239 01:223036 --:------ 0009 003 000000
+    # .I --- 23:100224 --:------ 23:100224 0009 003 0100FF  # 2-zone ST9520C
+    # .I --- 10:040239 01:223036 --:------ 0009 003 000000
 
     def _parser(seqx) -> dict:
-        assert seqx[:2] in (F9, FC) or int(seqx[:2], 16) < msg._gwy.config.max_zones
+        assert seqx[:2] in (F9, FC) or int(seqx[:2], 16) < 16
         return {
             SZ_DOMAIN_ID if seqx[:1] == "F" else SZ_ZONE_IDX: seqx[:2],
             "failsafe_enabled": {"00": False, "01": True}.get(seqx[2:4]),
@@ -390,7 +391,7 @@ def parser_000a(payload, msg) -> dict | list:
 
 @parser_decorator  # zone_devices
 def parser_000c(payload, msg) -> dict:
-    #  I --- 34:092243 --:------ 34:092243 000C 018 00-0A-7F-FFFFFF 00-0F-7F-FFFFFF 00-10-7F-FFFFFF  # noqa: E501
+    # .I --- 34:092243 --:------ 34:092243 000C 018 00-0A-7F-FFFFFF 00-0F-7F-FFFFFF 00-10-7F-FFFFFF  # noqa: E501
     # RP --- 01:145038 18:013393 --:------ 000C 006 00-00-00-10DAFD
     # RP --- 01:145038 18:013393 --:------ 000C 012 01-00-00-10DAF5 01-00-00-10DAFB
 
@@ -415,16 +416,14 @@ def parser_000c(payload, msg) -> dict:
             assert int(seqx, 16) < 1, f"invalid _idx: '{seqx}' (0x02)"
             return {SZ_DOMAIN_ID: FC}
 
-        assert (
-            int(seqx, 16) < msg._gwy.config.max_zones
-        ), f"invalid zone_idx: '{seqx}' (0x03)"
+        assert int(seqx, 16) < 16, f"invalid zone_idx: '{seqx}' (0x03)"
         return {SZ_ZONE_IDX: seqx}
 
     def _parser(seqx) -> dict:  # TODO: assumption that all id/idx are same is wrong!
         assert (
             seqx[:2] == payload[:2]
         ), f"idx != {payload[:2]} (seqx = {seqx}), short={is_short_000C(payload)}"
-        assert int(seqx[:2], 16) < msg._gwy.config.max_zones
+        assert int(seqx[:2], 16) < 16
         assert seqx[4:6] == "7F" or seqx[6:] != "F" * 6
         return {hex_id_to_dev_id(seqx[6:12]): seqx[4:6]}
 
@@ -574,14 +573,14 @@ def parser_0404(payload, msg) -> dict:
     # RP --- 01:037519 30:185469 --:------ 0404 014 00-230008-07-0303-13F...
 
     # Write a Zone schedule...
-    #  W --- 30:042165 01:076010 --:------ 0404 048 08-200808-29-0104-688...
-    #  I --- 01:076010 30:042165 --:------ 0404 007 08-200808-29-0104
-    #  W --- 30:042165 01:076010 --:------ 0404 048 08-200808-29-0204-007...
-    #  I --- 01:076010 30:042165 --:------ 0404 007 08-200808-29-0204
-    #  W --- 30:042165 01:076010 --:------ 0404 048 08-200808-29-0304-8DD...
-    #  I --- 01:076010 30:042165 --:------ 0404 007 08-200808-29-0304
-    #  W --- 30:042165 01:076010 --:------ 0404 048 08-200808-11-0404-970...
-    #  I --- 01:076010 30:042165 --:------ 0404 007 08-200808-11-0400
+    # .W --- 30:042165 01:076010 --:------ 0404 048 08-200808-29-0104-688...
+    # .I --- 01:076010 30:042165 --:------ 0404 007 08-200808-29-0104
+    # .W --- 30:042165 01:076010 --:------ 0404 048 08-200808-29-0204-007...
+    # .I --- 01:076010 30:042165 --:------ 0404 007 08-200808-29-0204
+    # .W --- 30:042165 01:076010 --:------ 0404 048 08-200808-29-0304-8DD...
+    # .I --- 01:076010 30:042165 --:------ 0404 007 08-200808-29-0304
+    # .W --- 30:042165 01:076010 --:------ 0404 048 08-200808-11-0404-970...
+    # .I --- 01:076010 30:042165 --:------ 0404 007 08-200808-11-0400
 
     # RP --- 01:145038 18:013393 --:------ 0404 007 00-230008-00-01FF  # no schedule
 
@@ -623,7 +622,7 @@ def parser_0418(payload, msg) -> dict:
     # RP --- 01:145038 18:013393 --:------ 0418 022 000000B0000000000000000000007FFFFF7000000000  # noqa: E501
     # RP --- 01:145038 18:013393 --:------ 0418 022 000036B0010000000000108000007FFFFF7000000000  # noqa: E501
     # RP --- 01:145038 18:013393 --:------ 0418 022 000000B00401010000008694A3CC7FFFFF70000ECC8A  # noqa: E501
-    #  I --- 01:037519 --:------ 01:037519 0418 022 000000B0050000000000239581877FFFFF7000000001  # Evotouch Battery Error  # noqa: E501
+    # .I --- 01:037519 --:------ 01:037519 0418 022 000000B0050000000000239581877FFFFF7000000001  # Evotouch Battery Error  # noqa: E501
     # RP --- 01:037519 18:140805 --:------ 0418 022 004024B0060006000000CB94A112FFFFFF70007AD47D  # noqa: E501
     #                                                 0     0   1     1            3        3
     #                                                 2     8   2     8            0        8
@@ -638,7 +637,7 @@ def parser_0418(payload, msg) -> dict:
         assert payload[8:10] in FAULT_TYPE, f"fault type: {payload[8:10]}"
         assert payload[12:14] in FAULT_DEVICE_CLASS, f"device class: {payload[12:14]}"
         # 1C: 'Comms fault, Actuator': seen with boiler relays
-        assert int(payload[10:12], 16) < msg._gwy.config.max_zones or (
+        assert int(payload[10:12], 16) < 16 or (
             payload[10:12] in ("1C", F6, F9, FA, FC)
         ), f"domain id: {payload[10:12]}"
     except AssertionError as exc:
@@ -661,11 +660,7 @@ def parser_0418(payload, msg) -> dict:
         result[SZ_DEVICE_CLASS] = DEV_ROLE_MAP[DEV_ROLE.HT1]  # speculative
 
     if payload[12:14] != "00":  # TODO: Controller
-        key_name = (
-            SZ_ZONE_IDX
-            if int(payload[10:12], 16) < msg._gwy.config.max_zones
-            else SZ_DOMAIN_ID
-        )
+        key_name = SZ_ZONE_IDX if int(payload[10:12], 16) < 16 else SZ_DOMAIN_ID
         result.update({key_name: payload[10:12]})
 
     if payload[38:] == "000002":  # "00:000002 for Unknown?
@@ -686,15 +681,15 @@ def parser_0418(payload, msg) -> dict:
 
 @parser_decorator  # unknown_042f, from STA, VMS
 def parser_042f(payload, msg) -> dict:
-    #  I --- 34:064023 --:------ 34:064023 042F 008 00-0000-0023-0023-F5
-    #  I --- 34:064023 --:------ 34:064023 042F 008 00-0000-0024-0024-F5
-    #  I --- 34:064023 --:------ 34:064023 042F 008 00-0000-0025-0025-F5
-    #  I --- 34:064023 --:------ 34:064023 042F 008 00-0000-0026-0026-F5
-    #  I --- 34:092243 --:------ 34:092243 042F 008 00-0001-0021-0022-01
-    #  I     34:011469 --:------ 34:011469 042F 008 00-0001-0003-0004-BC
+    # .I --- 34:064023 --:------ 34:064023 042F 008 00-0000-0023-0023-F5
+    # .I --- 34:064023 --:------ 34:064023 042F 008 00-0000-0024-0024-F5
+    # .I --- 34:064023 --:------ 34:064023 042F 008 00-0000-0025-0025-F5
+    # .I --- 34:064023 --:------ 34:064023 042F 008 00-0000-0026-0026-F5
+    # .I --- 34:092243 --:------ 34:092243 042F 008 00-0001-0021-0022-01
+    # .I   34:011469 --:------ 34:011469 042F 008 00-0001-0003-0004-BC
 
-    #  I --- 32:168090 --:------ 32:168090 042F 009 00-0000100F00105050
-    #  I --- 32:166025 --:------ 32:166025 042F 009 00-050E0B0C00111470
+    # .I --- 32:168090 --:------ 32:168090 042F 009 00-0000100F00105050
+    # .I --- 32:166025 --:------ 32:166025 042F 009 00-050E0B0C00111470
 
     return {
         "counter_1": f"0x{payload[2:6]}",
@@ -706,7 +701,7 @@ def parser_042f(payload, msg) -> dict:
 
 @parser_decorator  # TODO: unknown_0b04, from THM (only when its a CTL?)
 def parser_0b04(payload, msg) -> dict:
-    #  I --- --:------ --:------ 12:207082 0B04 002 00C8  # batch of 3, every 24h
+    # .I --- --:------ --:------ 12:207082 0B04 002 00C8  # batch of 3, every 24h
 
     return {
         f"{SZ_UNKNOWN}_1": payload[2:],
@@ -715,8 +710,8 @@ def parser_0b04(payload, msg) -> dict:
 
 @parser_decorator  # mixvalve_config (zone), FAN
 def parser_1030(payload, msg) -> dict:
-    #  I --- 01:145038 --:------ 01:145038 1030 016 0A-C80137-C9010F-CA0196-CB0100-CC0101
-    #  I --- --:------ --:------ 12:144017 1030 016 01-C80137-C9010F-CA0196-CB010F-CC0101
+    # .I --- 01:145038 --:------ 01:145038 1030 016 0A-C80137-C9010F-CA0196-CB0100-CC0101
+    # .I --- --:------ --:------ 12:144017 1030 016 01-C80137-C9010F-CA0196-CB010F-CC0101
     # RP --- 32:155617 18:005904 --:------ 1030 007 00-200100-21011F
 
     def _parser(seqx) -> dict:
@@ -757,7 +752,7 @@ def parser_1060(payload, msg) -> dict:
 
     return {
         "battery_low": payload[4:] == "00",
-        "battery_level": percent(payload[2:4]),
+        "battery_level": percent_from_hex(payload[2:4]),
     }
 
 
@@ -789,7 +784,7 @@ def parser_1098(payload, msg) -> dict:
     return {
         f"_{SZ_PAYLOAD}": payload,
         f"_{SZ_VALUE}": {"00": False, "C8": True}.get(
-            payload[2:], percent(payload[2:])
+            payload[2:], percent_from_hex(payload[2:])
         ),
     }
 
@@ -844,7 +839,7 @@ def parser_10b0(payload, msg) -> dict:
     return {
         f"_{SZ_PAYLOAD}": payload,
         f"_{SZ_VALUE}": {"00": False, "C8": True}.get(
-            payload[2:], percent(payload[2:])
+            payload[2:], percent_from_hex(payload[2:])
         ),
     }
 
@@ -870,7 +865,7 @@ def parser_10d0(payload, msg) -> dict:
     if msg.len >= 3:
         result.update({"days_lifetime": int(payload[4:6], 16)})
     if msg.len >= 4:
-        result.update({"percent_remaining": percent(payload[6:8])})
+        result.update({"percent_remaining": percent_from_hex(payload[6:8])})
 
     return result
 
@@ -1002,7 +997,7 @@ def parser_1260(payload, msg) -> dict:
 def parser_1280(payload, msg) -> dict:
     # educated guess - this packet never seen in the wild
 
-    rh = percent(payload[2:4], high_res=False)
+    rh = percent_from_hex(payload[2:4], high_res=False)
     if msg.len == 2:
         return {SZ_OUTDOOR_HUMIDITY: rh}
 
@@ -1021,7 +1016,7 @@ def parser_1290(payload, msg) -> dict:
 
 @parser_decorator  # co2_level
 def parser_1298(payload, msg) -> dict:
-    #  I --- 37:258565 --:------ 37:258565 1298 003 0007D0
+    # .I --- 37:258565 --:------ 37:258565 1298 003 0007D0
     FAULT_CODES_CO2 = {
         "80": "sensor short circuit",
         "81": "sensor open",
@@ -1032,7 +1027,7 @@ def parser_1298(payload, msg) -> dict:
     if fault := FAULT_CODES_CO2.get(payload[:2]):
         return {"sensor_fault": fault}
 
-    return {SZ_CO2_LEVEL: double(payload[2:])}
+    return {SZ_CO2_LEVEL: double_from_hex(payload[2:])}
 
 
 @parser_decorator  # indoor_humidity
@@ -1065,7 +1060,7 @@ def parser_12a0(payload, msg) -> dict:
     # if (fault := FAULT_CODES_TEMP.get(payload[2:4])):
     #     return {"sensor_fault": fault}
 
-    rh = percent(payload[2:4], high_res=False)
+    rh = percent_from_hex(payload[2:4], high_res=False)
     if msg.len == 2:
         return {SZ_INDOOR_HUMIDITY: rh}
 
@@ -1112,7 +1107,7 @@ def parser_12c8(payload, msg) -> dict:
     # 04:51:41.192 078  I --- 37:261128 --:------ 37:261128 12C8 003 009540
 
     return {
-        SZ_AIR_QUALITY: percent(payload[2:4]),  # 31DA[2:4]
+        SZ_AIR_QUALITY: percent_from_hex(payload[2:4]),  # 31DA[2:4]
         SZ_AIR_QUALITY_BASE: int(payload[4:6], 16),  # 31DA[4:6]
     }
 
@@ -1239,19 +1234,19 @@ def parser_1F70(payload, msg) -> dict:
 
 @parser_decorator  # rf_bind
 def parser_1fc9(payload, msg) -> list:
-    #  I --- 01:145038 --:------ 01:145038 1FC9 012 F6-2D49-06368E F6-1FC9-06368E
+    # .I --- 01:145038 --:------ 01:145038 1FC9 012 F6-2D49-06368E F6-1FC9-06368E
 
-    #  I is missing?
-    #  W --- 10:048122 01:145038 --:------ 1FC9 006 003EF028BBFA
-    #  I --- 01:145038 10:048122 --:------ 1FC9 006 00FFFF06368E
+    # .I is missing?
+    # .W --- 10:048122 01:145038 --:------ 1FC9 006 003EF028BBFA
+    # .I --- 01:145038 10:048122 --:------ 1FC9 006 00FFFF06368E
 
-    #  I --- 07:045960 --:------ 07:045960 1FC9 012 0012601CB388001FC91CB388
-    #  W --- 01:145038 07:045960 --:------ 1FC9 006 0010A006368E
-    #  I --- 07:045960 01:145038 --:------ 1FC9 006 0012601CB388
+    # .I --- 07:045960 --:------ 07:045960 1FC9 012 0012601CB388001FC91CB388
+    # .W --- 01:145038 07:045960 --:------ 1FC9 006 0010A006368E
+    # .I --- 07:045960 01:145038 --:------ 1FC9 006 0012601CB388
 
-    #  I --- 01:145038 --:------ 01:145038 1FC9 018 FA000806368EFC3B0006368EFA1FC906368E
-    #  W --- 13:081807 01:145038 --:------ 1FC9 006 003EF0353F8F
-    #  I --- 01:145038 13:081807 --:------ 1FC9 006 00FFFF06368E
+    # .I --- 01:145038 --:------ 01:145038 1FC9 018 FA000806368EFC3B0006368EFA1FC906368E
+    # .W --- 13:081807 01:145038 --:------ 1FC9 006 003EF0353F8F
+    # .I --- 01:145038 13:081807 --:------ 1FC9 006 00FFFF06368E
 
     # this is an array of codes
     # 049  I --- 01:145038 --:------ 01:145038 1FC9 018 07-0008-06368E FC-3B00-06368E                07-1FC9-06368E  # noqa: E501
@@ -1287,7 +1282,7 @@ def parser_1fc9(payload, msg) -> list:
             FC,
             FF,
         ):  # or: not in DOMAIN_TYPE_MAP: ??
-            assert int(seqx[:2], 16) < msg._gwy.config.max_zones
+            assert int(seqx[:2], 16) < 16
         return [seqx[:2], seqx[2:6], hex_id_to_dev_id(seqx[6:])]
 
     if payload == "00":
@@ -1307,7 +1302,7 @@ def parser_1fc9(payload, msg) -> list:
 
 @parser_decorator  # unknown_1fca, HVAC?
 def parser_1fca(payload, msg) -> list:
-    #  W --- 30:248208 34:021943 --:------ 1FCA 009 00-01FF-7BC990-FFFFFF  # sent x2
+    # .W --- 30:248208 34:021943 --:------ 1FCA 009 00-01FF-7BC990-FFFFFF  # sent x2
 
     return {
         f"_{SZ_UNKNOWN}_0": payload[:2],
@@ -1335,7 +1330,7 @@ def parser_1fd4(payload, msg) -> dict:
 @parser_decorator  # now_next_setpoint - Programmer/Hometronics
 def parser_2249(payload, msg) -> dict:
     # see: https://github.com/jrosser/honeymon/blob/master/decoder.cpp#L357-L370
-    #  I --- 23:100224 --:------ 23:100224 2249 007 00-7EFF-7EFF-FFFF
+    # .I --- 23:100224 --:------ 23:100224 2249 007 00-7EFF-7EFF-FFFF
 
     def _parser(seqx) -> dict:
         minutes = int(seqx[10:], 16)
@@ -1364,11 +1359,11 @@ def parser_2249(payload, msg) -> dict:
 def parser_22b0(payload, msg) -> dict:
     # Seen on Orcon: see 1470, 1F70, 22B0
 
-    #  W --- 37:171871 32:155617 --:------ 22B0 002 0005  # enable
-    #  I --- 32:155617 37:171871 --:------ 22B0 002 0005
+    # .W --- 37:171871 32:155617 --:------ 22B0 002 0005  # enable
+    # .I --- 32:155617 37:171871 --:------ 22B0 002 0005
 
-    #  W --- 37:171871 32:155617 --:------ 22B0 002 0006  # disable
-    #  I --- 32:155617 37:171871 --:------ 22B0 002 0006
+    # .W --- 37:171871 32:155617 --:------ 22B0 002 0006  # disable
+    # .I --- 32:155617 37:171871 --:------ 22B0 002 0006
 
     return {
         "enabled": {"06": False, "05": True}.get(payload[2:4]),
@@ -1416,20 +1411,20 @@ def parser_22d9(payload, msg) -> dict:
 @parser_decorator  # fan_speed (switch_mode), HVAC
 def parser_22f1(payload, msg) -> dict:
     # Orcon wireless remote 15RF
-    # I --- 37:171871 32:155617 --:------ 22F1 003 000007  # Absent mode  // Afwezig (absence mode, aka: weg/away) - low & doesn't respond to sensors
-    # I --- 37:171871 32:155617 --:------ 22F1 003 000107  # Mode 1: Low  // Stand 1 (position low)
-    # I --- 37:171871 32:155617 --:------ 22F1 003 000207  # Mode 2: Med  // Stand 2 (position med)
-    # I --- 37:171871 32:155617 --:------ 22F1 003 000307  # Mode 3: High // Stand 3 (position high)
-    # I --- 37:171871 32:155617 --:------ 22F1 003 000407  # Auto
-    # I --- 37:171871 32:155617 --:------ 22F1 003 000507  # Auto
-    # I --- 37:171871 32:155617 --:------ 22F1 003 000607  # Party/boost
-    # I --- 37:171871 32:155617 --:------ 22F1 003 000707  # Off
+    # .I --- 37:171871 32:155617 --:------ 22F1 003 000007  # Absent mode  // Afwezig (absence mode, aka: weg/away) - low & doesn't respond to sensors
+    # .I --- 37:171871 32:155617 --:------ 22F1 003 000107  # Mode 1: Low  // Stand 1 (position low)
+    # .I --- 37:171871 32:155617 --:------ 22F1 003 000207  # Mode 2: Med  // Stand 2 (position med)
+    # .I --- 37:171871 32:155617 --:------ 22F1 003 000307  # Mode 3: High // Stand 3 (position high)
+    # .I --- 37:171871 32:155617 --:------ 22F1 003 000407  # Auto
+    # .I --- 37:171871 32:155617 --:------ 22F1 003 000507  # Auto
+    # .I --- 37:171871 32:155617 --:------ 22F1 003 000607  # Party/boost
+    # .I --- 37:171871 32:155617 --:------ 22F1 003 000707  # Off
 
-    #  I 015 --:------ --:------ 39:159057 22F1 003 000004  # TBA: off/standby?
-    #  I 015 --:------ --:------ 39:159057 22F1 003 000104  # TBA: trickle/min-speed?
-    #  I 015 --:------ --:------ 39:159057 22F1 003 000204  # low
-    #  I 016 --:------ --:------ 39:159057 22F1 003 000304  # medium
-    #  I 017 --:------ --:------ 39:159057 22F1 003 000404  # high (aka boost if timer)
+    # .I 015 --:------ --:------ 39:159057 22F1 003 000004  # TBA: off/standby?
+    # .I 015 --:------ --:------ 39:159057 22F1 003 000104  # TBA: trickle/min-speed?
+    # .I 015 --:------ --:------ 39:159057 22F1 003 000204  # low
+    # .I 016 --:------ --:------ 39:159057 22F1 003 000304  # medium
+    # .I 017 --:------ --:------ 39:159057 22F1 003 000404  # high (aka boost if timer)
 
     # Scheme x: 0|x standby/off, 1|x min, 2+|x rate as % of max (Itho?)
     # Scheme 4: 0|4 standby/off, 1|4 auto, 2|4 low, 3|4 med, 4|4 high/boost
@@ -1479,21 +1474,21 @@ def parser_22f1(payload, msg) -> dict:
         _LOGGER.warning(f"{msg!r} < {_INFORM_DEV_MSG} ({exc})")
 
     return {
+        SZ_FAN_MODE: _22F1_FAN_MODE.get(payload[2:4], f"unknown_{payload[2:4]}"),
         "_mode_idx": f"{int(payload[2:4], 16) & 0x07:02X}",
         "_mode_max": payload[4:6] or None,
-        "fan_mode": _22F1_FAN_MODE.get(payload[2:4], f"unknown_{payload[2:4]}"),
         "_scheme": _22f1_scheme,
     }
 
 
 @parser_decorator  # switch_boost, HVAC
 def parser_22f3(payload, msg) -> dict:
-    #  I 019 --:------ --:------ 39:159057 22F3 003 00000A  # 10 mins
-    #  I 022 --:------ --:------ 39:159057 22F3 003 000014  # 20 mins
-    #  I 026 --:------ --:------ 39:159057 22F3 003 00001E  # 30 mins
-    #  I --- 29:151550 29:237552 --:------ 22F3 007 00023C-0304-0000  # 60 mins
-    #  I --- 29:162374 29:237552 --:------ 22F3 007 00020F-0304-0000  # 15 mins
-    #  I --- 29:162374 29:237552 --:------ 22F3 007 00020F-0304-0000  # 15 mins
+    # .I 019 --:------ --:------ 39:159057 22F3 003 00000A  # 10 mins
+    # .I 022 --:------ --:------ 39:159057 22F3 003 000014  # 20 mins
+    # .I 026 --:------ --:------ 39:159057 22F3 003 00001E  # 30 mins
+    # .I --- 29:151550 29:237552 --:------ 22F3 007 00023C-0304-0000  # 60 mins
+    # .I --- 29:162374 29:237552 --:------ 22F3 007 00020F-0304-0000  # 15 mins
+    # .I --- 29:162374 29:237552 --:------ 22F3 007 00020F-0304-0000  # 15 mins
 
     # NOTE: for boost timer for high
     try:
@@ -1550,12 +1545,12 @@ def parser_22f7(payload, msg) -> dict:
     # RQ --- 37:171871 32:155617 --:------ 22F7 001 00
     # RP --- 32:155617 37:171871 --:------ 22F7 003 00FF00  # alse: 000000, 00C8C8
 
-    #  W --- 37:171871 32:155617 --:------ 22F7 003 0000EF  # bypass off
-    #  I --- 32:155617 37:171871 --:------ 22F7 003 000000
-    #  W --- 37:171871 32:155617 --:------ 22F7 003 00C8EF  # bypass on
-    #  I --- 32:155617 37:171871 --:------ 22F7 003 00C800
-    #  W --- 37:171871 32:155617 --:------ 22F7 003 00FFEF  # bypass auto
-    #  I --- 32:155617 37:171871 --:------ 22F7 003 00FFC8
+    # .W --- 37:171871 32:155617 --:------ 22F7 003 0000EF  # bypass off
+    # .I --- 32:155617 37:171871 --:------ 22F7 003 000000
+    # .W --- 37:171871 32:155617 --:------ 22F7 003 00C8EF  # bypass on
+    # .I --- 32:155617 37:171871 --:------ 22F7 003 00C800
+    # .W --- 37:171871 32:155617 --:------ 22F7 003 00FFEF  # bypass auto
+    # .I --- 32:155617 37:171871 --:------ 22F7 003 00FFC8
 
     result = {
         "bypass_mode": {"00": "off", "C8": "on", "FF": "auto"}.get(payload[2:4]),
@@ -1590,7 +1585,7 @@ def parser_2349(payload, msg) -> dict:
     # RQ --- 34:225071 30:258557 --:------ 2349 001 00
     # RP --- 30:258557 34:225071 --:------ 2349 013 007FFF00FFFFFFFFFFFFFFFFFF
     # RP --- 30:253184 34:010943 --:------ 2349 013 00064000FFFFFF00110E0507E5
-    #  I --- 10:067219 --:------ 10:067219 2349 004 00000001
+    # .I --- 10:067219 --:------ 10:067219 2349 004 00000001
 
     if msg.verb == RQ and msg.len <= 2:  # some RQs have a payload (why?)
         return {}
@@ -1657,7 +1652,7 @@ def parser_2401(payload, msg) -> dict:
         SZ_PAYLOAD: payload,
         "_value_2": int(payload[4:6], 0x10),
         "_flags_2": flag8(payload[4:6]),
-        "_percent_3": percent(payload[6:]),
+        "_percent_3": percent_from_hex(payload[6:]),
     }
 
 
@@ -1693,7 +1688,7 @@ def parser_2411(payload, msg) -> dict:
     _2411_DATA_TYPES = {
         "00": (2, counter),  # 4E (0-1), 54 (15-60)
         "01": (2, centile),  # 52 (0.0-25.0) (%)
-        "0F": (2, percent),  # xx (0.0-1.0) (%)
+        "0F": (2, percent_from_hex),  # xx (0.0-1.0) (%)
         "10": (4, counter),  # 31 (0-1800) (days)
         "92": (4, temp_from_hex),  # 75 (0-30) (C)
     }  # TODO: _2411_TYPES.get(payload[8:10], (8, no_op))
@@ -1756,8 +1751,8 @@ def parser_2d49(payload, msg) -> dict:
 def parser_2e04(payload, msg) -> dict:
     # if msg.verb == W_:
 
-    #  I --— 01:020766 --:------ 01:020766 2E04 016 FFFFFFFFFFFFFF0007FFFFFFFFFFFF04  # Manual          # noqa: E501
-    #  I --— 01:020766 --:------ 01:020766 2E04 016 FFFFFFFFFFFFFF0000FFFFFFFFFFFF04  # Automatic/times # noqa: E501
+    # .I --— 01:020766 --:------ 01:020766 2E04 016 FFFFFFFFFFFFFF0007FFFFFFFFFFFF04  # Manual          # noqa: E501
+    # .I --— 01:020766 --:------ 01:020766 2E04 016 FFFFFFFFFFFFFF0000FFFFFFFFFFFF04  # Automatic/times # noqa: E501
 
     if msg.len == 8:  # evohome
         assert payload[:2] in SYS_MODE_MAP, f"Unknown system mode: {payload[:2]}"
@@ -1824,16 +1819,16 @@ def parser_3110(payload, msg) -> dict:
 
     return {
         f"_{SZ_UNKNOWN}_1": payload[2:4],
-        "_percent_2": percent(payload[4:6]),
+        "_percent_2": percent_from_hex(payload[4:6]),
         "_value_3": payload[6:],
     }
 
 
 @parser_decorator  # unknown_3120, from STA, FAN
 def parser_3120(payload, msg) -> dict:
-    #  I --- 34:136285 --:------ 34:136285 3120 007 0070B0000000FF  # every ~3:45:00!
+    # .I --- 34:136285 --:------ 34:136285 3120 007 0070B0000000FF  # every ~3:45:00!
     # RP --- 20:008749 18:142609 --:------ 3120 007 0070B000009CFF
-    #  I --- 37:258565 --:------ 37:258565 3120 007 0080B0010003FF
+    # .I --- 37:258565 --:------ 37:258565 3120 007 0080B0010003FF
 
     try:
         assert payload[:2] == "00", f"byte 0: {payload[:2]}"
@@ -1888,7 +1883,7 @@ def parser_3150(payload, msg) -> dict | list:
     # event-driven, and periodically; FC domain is maximum of all zones
     # TODO: all have a valid domain will UFC/CTL respond to an RQ, for FC, for a zone?
 
-    #  I --- 04:136513 --:------ 01:158182 3150 002 01CA < often seen CA, artefact?
+    # .I --- 04:136513 --:------ 01:158182 3150 002 01CA < often seen CA, artefact?
 
     def complex_idx(seqx, msg) -> dict:
         # assert seqx[:2] == FC or (int(seqx[:2], 16) < MAX_ZONES)  # <5, 8 for UFC
@@ -1907,7 +1902,7 @@ def parser_3150(payload, msg) -> dict | list:
     return valve_demand(payload[2:])  # TODO: check UFC/FC is == CTL/FC
 
 
-@parser_decorator  # ventilation state basic, HVAC
+@parser_decorator  # fan state (basic), HVAC
 def parser_31d9(payload, msg) -> dict:
     # NOTE: I have a suspicion that Itho use 0x00-C8 for %, whilst Nuaire use 0x00-64
     try:
@@ -1919,18 +1914,16 @@ def parser_31d9(payload, msg) -> dict:
 
     bitmap = int(payload[2:4], 16)
 
-    # NOTE: 31D9[4:6] is fan_speed (itho?) *or* fan_mode (orcon?)
-    # TODO: 31D9[4:6] == 31DA[38:40] in some systems
-    # TODO: 31D9[4:6] == 31DA[36:38]??
+    # NOTE: 31D9[4:6] is fan_rate (itho?) *or* fan_mode (orcon?)
     result = {
-        SZ_EXHAUST_FAN_SPEED: percent(payload[4:6], high_res=True),
-        "fan_mode": payload[4:6],
+        SZ_EXHAUST_FAN_SPEED: percent_from_hex(payload[4:6], high_res=True),  # itho
+        SZ_FAN_MODE: payload[4:6],  # orcon
         "passive": bool(bitmap & 0x02),
         "damper_only": bool(bitmap & 0x04),
         "filter_dirty": bool(bitmap & 0x20),
         "frost_cycle": bool(bitmap & 0x40),
         "has_fault": bool(bitmap & 0x80),
-        "flags": flag8(payload[2:4]),
+        "_flags": flag8(payload[2:4]),
     }
 
     if msg.len == 3:  # usu: I -->20: (no seq#)
@@ -1941,7 +1934,7 @@ def parser_31d9(payload, msg) -> dict:
     except AssertionError as exc:
         _LOGGER.warning(f"{msg!r} < {_INFORM_DEV_MSG} ({exc})")
 
-    # result.update({f"_{SZ_UNKNOWN}_3": payload[6:8]})
+    result.update({f"_{SZ_UNKNOWN}_3": payload[6:8]})
 
     if msg.len == 4:  # usu: I -->20: (no seq#)
         return result
@@ -1954,49 +1947,14 @@ def parser_31d9(payload, msg) -> dict:
 
     return {
         **result,
-        # f"_{SZ_UNKNOWN}_4": payload[8:32],
-        # f"{SZ_UNKNOWN}_16": payload[32:],
+        f"_{SZ_UNKNOWN}_4": payload[8:32],
+        f"{SZ_UNKNOWN}_16": payload[32:],
     }
 
 
-@parser_decorator  # ventilation state extended, HVAC
+@parser_decorator  # ventilation state (extended), HVAC
 def parser_31da(payload, msg) -> dict:
 
-    _31DA_FAN_INFO = {
-        0x00: "off",
-        0x01: "speed 1",
-        0x02: "speed 2",
-        0x03: "speed 3",
-        0x04: "speed 4",
-        0x05: "speed 5",
-        0x06: "speed 6",
-        0x07: "speed 7",
-        0x08: "speed 8",
-        0x09: "speed 9",
-        0x0A: "speed 10",
-        0x0B: "speed 1 temporary override",
-        0x0C: "speed 2 temporary override",
-        0x0D: "speed 3 temporary override",
-        0x0E: "speed 4 temporary override",
-        0x0F: "speed 5 temporary override",
-        0x10: "speed 6 temporary override",
-        0x11: "speed 7 temporary override",
-        0x12: "speed 8 temporary override",
-        0x13: "speed 9 temporary override",
-        0x14: "speed 10 temporary override",
-        0x15: "away",
-        0x16: "absolute minimum",  # trickle?
-        0x17: "absolute maximum",  # boost?
-        0x18: "auto",
-        0x19: "-unknown 0x19-",
-        0x1A: "-unknown 0x1A-",
-        0x1B: "-unknown 0x1B-",
-        0x1C: "-unknown 0x1C-",
-        0x1D: "-unknown 0x1D-",
-        0x1E: "-unknown 0x1E-",
-        0x1F: "-unknown 0x1F-",
-    }
-
     try:
         # assert (
         #     int(payload[2:4], 16) <= 200
@@ -2037,30 +1995,32 @@ def parser_31da(payload, msg) -> dict:
     # 12 Fan supply speed (%)                 SZ_SUPPLY_FAN_SPEED
     # 13 Remaining after run time (humidity scenario) (min.)  SZ_REMAINING_TIME
     # 14 Preheater control (MaxComfort) (%)   SZ_PRE_HEAT
-    # 16 Actual supply flow rate (m3/h)       SZ_SUPPLY_FLOW (Orcon is m3/h, others L/s)
+    # 16 Actual supply flow rate (m3/h)       SZ_SUPPLY_FLOW (Orcon is m3/h, data is L/s)
     # 17 Current discharge flow rate (m3/h)   SZ_EXHAUST_FLOW
 
     return {
-        SZ_EXHAUST_FAN_SPEED: percent(payload[38:40]),  # maybe 31D9[4:6] for some?
+        SZ_EXHAUST_FAN_SPEED: percent_from_hex(
+            payload[38:40]
+        ),  # maybe 31D9[4:6] for some?
         SZ_FAN_INFO: _31DA_FAN_INFO[int(payload[36:38], 16) & 0x1F],  # 22F3-ish
-        SZ_REMAINING_TIME: double(payload[42:46]),  # mins, 22F3[2:6]
+        SZ_REMAINING_TIME: double_from_hex(payload[42:46]),  # mins, 22F3[2:6]
         #
-        SZ_AIR_QUALITY: percent(payload[2:4]),  # 12C8[2:4]
+        SZ_AIR_QUALITY: percent_from_hex(payload[2:4]),  # 12C8[2:4]
         SZ_AIR_QUALITY_BASE: int(payload[4:6], 16),  # 12C8[4:6]
-        SZ_CO2_LEVEL: double(payload[6:10]),  # ppm, 1298[2:6]
-        SZ_INDOOR_HUMIDITY: percent(payload[10:12], high_res=False),  # 12A0?
-        SZ_OUTDOOR_HUMIDITY: percent(payload[12:14], high_res=False),
-        SZ_EXHAUST_TEMPERATURE: double(payload[14:18], factor=100),
-        SZ_SUPPLY_TEMPERATURE: double(payload[18:22], factor=100),
-        SZ_INDOOR_TEMPERATURE: double(payload[22:26], factor=100),
-        SZ_OUTDOOR_TEMPERATURE: double(payload[26:30], factor=100),  # 1290?
+        SZ_CO2_LEVEL: double_from_hex(payload[6:10]),  # ppm, 1298[2:6]
+        SZ_INDOOR_HUMIDITY: percent_from_hex(payload[10:12], high_res=False),  # 12A0?
+        SZ_OUTDOOR_HUMIDITY: percent_from_hex(payload[12:14], high_res=False),
+        SZ_EXHAUST_TEMPERATURE: double_from_hex(payload[14:18], factor=100),
+        SZ_SUPPLY_TEMPERATURE: double_from_hex(payload[18:22], factor=100),
+        SZ_INDOOR_TEMPERATURE: double_from_hex(payload[22:26], factor=100),
+        SZ_OUTDOOR_TEMPERATURE: double_from_hex(payload[26:30], factor=100),  # 1290?
         SZ_SPEED_CAP: int(payload[30:34], 16),
-        SZ_BYPASS_POSITION: percent(payload[34:36]),
-        SZ_SUPPLY_FAN_SPEED: percent(payload[40:42]),
-        SZ_POST_HEAT: percent(payload[46:48], high_res=False),
-        SZ_PRE_HEAT: percent(payload[48:50], high_res=False),
-        SZ_SUPPLY_FLOW: double(payload[50:54], factor=100),  # L/sec
-        SZ_EXHAUST_FLOW: double(payload[54:58], factor=100),  # L/sec
+        SZ_BYPASS_POSITION: percent_from_hex(payload[34:36]),
+        SZ_SUPPLY_FAN_SPEED: percent_from_hex(payload[40:42]),
+        SZ_POST_HEAT: percent_from_hex(payload[46:48], high_res=False),
+        SZ_PRE_HEAT: percent_from_hex(payload[48:50], high_res=False),
+        SZ_SUPPLY_FLOW: double_from_hex(payload[50:54], factor=100),  # L/sec
+        SZ_EXHAUST_FLOW: double_from_hex(payload[54:58], factor=100),  # L/sec
     }
 
 
@@ -2106,18 +2066,18 @@ def parser_31e0(payload, msg) -> dict:
         break;
     }
     """
-    #  I --- 29:146052 32:023459 --:------ 31E0 003 00-00-00
-    #  I --- 29:146052 32:023459 --:------ 31E0 003 00-00-C8
+    # .I --- 29:146052 32:023459 --:------ 31E0 003 00-00-00
+    # .I --- 29:146052 32:023459 --:------ 31E0 003 00-00-C8
 
-    #  I --- 32:168240 30:079129 --:------ 31E0 004 00-00-00-FF
-    #  I --- 32:168240 30:079129 --:------ 31E0 004 00-00-00-FF
-    #  I --- 32:166025 --:------ 30:079129 31E0 004 00-00-00-00
+    # .I --- 32:168240 30:079129 --:------ 31E0 004 00-00-00-FF
+    # .I --- 32:168240 30:079129 --:------ 31E0 004 00-00-00-FF
+    # .I --- 32:166025 --:------ 30:079129 31E0 004 00-00-00-00
 
-    #  I --- 32:168090 30:082155 --:------ 31E0 004 00-00-C8-00
-    #  I --- 37:258565 37:261128 --:------ 31E0 004 00-00-01-00
+    # .I --- 32:168090 30:082155 --:------ 31E0 004 00-00-C8-00
+    # .I --- 37:258565 37:261128 --:------ 31E0 004 00-00-01-00
 
     return {
-        "vent_demand": percent(payload[4:6]),
+        "vent_demand": percent_from_hex(payload[4:6]),
         "flags_1": payload[2:4],
         f"_{SZ_UNKNOWN}_3": payload[6:],
     }
@@ -2291,13 +2251,13 @@ def parser_3ef0(payload, msg) -> dict:
         # .I --- 13:023770 --:------ 13:023770 3EF0 003 00C8FF
         assert payload[2:4] in ("00", "C8"), f"byte 1: {payload[2:4]}"
         assert payload[4:6] == "FF", f"byte 2: {payload[4:6]}"
-        mod_level = percent(payload[2:4])
+        mod_level = percent_from_hex(payload[2:4])
 
     if msg.len >= 6:  # RP|OTB|006 (to RQ|CTL/HGI/RFG)
         # RP --- 10:105624 01:133689 --:------ 3EF0 006 0000100000FF
         # RP --- 10:105624 01:133689 --:------ 3EF0 006 003B100C00FF
         assert payload[4:6] in ("00", "10", "11"), f"byte 2: {payload[4:6]}"
-        mod_level = percent(payload[2:4], high_res=False)
+        mod_level = percent_from_hex(payload[2:4], high_res=False)
 
     result = {
         "modulation_level": mod_level,  # 0008[2:4], 3EF1[10:12]
@@ -2336,7 +2296,7 @@ def parser_3ef0(payload, msg) -> dict:
                 "_flags_6": flag8(payload[12:14]),
                 "ch_enabled": bool(int(payload[12:14], 0x10) & 1 << 0),
                 "ch_setpoint": int(payload[14:16], 0x10),
-                "max_rel_modulation": percent(payload[16:18], high_res=False),
+                "max_rel_modulation": percent_from_hex(payload[16:18], high_res=False),
             }
         )
 
@@ -2382,7 +2342,7 @@ def parser_3ef1(payload, msg) -> dict:
         # assert payload[6:10] in ("87B3", "9DFA", "DCE1", "E638", "F8F7") or (
         #     int(payload[6:10], 16) <= 7200
         # ), f"byte 3: {payload[6:10]}"
-        assert percent(payload[10:12]) in (0, 1), f"byte 5: {payload[10:12]}"
+        assert percent_from_hex(payload[10:12]) in (0, 1), f"byte 5: {payload[10:12]}"
 
     else:  # is OTB
         # assert (
@@ -2390,12 +2350,12 @@ def parser_3ef1(payload, msg) -> dict:
         # ), "doesn't match: " + r"^00[0-9A-F]{10}10"
         assert payload[2:6] == "7FFF", f"byte 1: {payload[2:6]}"
         assert payload[6:10] == "003C", f"byte 3: {payload[6:10]}"  # 60 seconds
-        assert percent(payload[10:12]) <= 1, f"byte 5: {payload[10:12]}"
+        assert percent_from_hex(payload[10:12]) <= 1, f"byte 5: {payload[10:12]}"
 
     cycle_countdown = None if payload[2:6] == "7FFF" else int(payload[2:6], 16)
 
     return {
-        "modulation_level": percent(payload[10:12]),  # 0008[2:4], 3EF0[2:4]
+        "modulation_level": percent_from_hex(payload[10:12]),  # 0008[2:4], 3EF0[2:4]
         "actuator_countdown": int(payload[6:10], 16),
         "cycle_countdown": cycle_countdown,
         f"_{SZ_UNKNOWN}_0": payload[12:],
