@@ -19,10 +19,7 @@ For socat, see:
   python client.py monitor /dev/pts/0
   cat packet.log | cut -d ' ' -f 2- | unix2dos > /dev/pts/1
 """
-
-# TODO: switch dtm from naive to aware
-# TODO: https://evohome-hackers.slack.com/archives/C02SYCLATSL/p1646997554178989
-# TODO: https://evohome-hackers.slack.com/archives/C02SYCLATSL/p1646998253052939
+from __future__ import annotations
 
 import asyncio
 import logging
@@ -37,67 +34,64 @@ from queue import Queue
 from string import printable  # ascii_letters, digits
 from time import perf_counter
 from types import SimpleNamespace
-from typing import Awaitable, Callable, Iterable, Optional
+from typing import Awaitable, Callable, Iterable, Optional, TypeVar
 
-from serial import SerialBase, SerialException, serial_for_url
-from serial_asyncio import SerialTransport as SerTransportAsync
+from serial import SerialBase, SerialException, serial_for_url  # type: ignore[import]
+from serial_asyncio import SerialTransport as SerTransportAsync  # type: ignore[import]
 
 from .address import HGI_DEV_ADDR, NON_DEV_ADDR, NUL_DEV_ADDR
-from .command import Command
-from .const import (
-    DEV_TYPE_MAP,
-    SZ_BACKOFF,
-    SZ_DEVICE_ID,
-    SZ_INBOUND,
-    SZ_OUTBOUND,
-    SZ_PRIORITY,
-    SZ_RETRIES,
-    SZ_TIMEOUT,
-    Priority,
-    __dev_mode__,
-)
+from .command import Command, Qos
+
+# skipcq: PY-W2000
+from .const import DEV_TYPE_MAP, SZ_DEVICE_ID, __dev_mode__
 from .exceptions import InvalidPacketError
 from .helpers import dt_now
 from .packet import Packet
 from .protocol import create_protocol_factory
-from .schema import SERIAL_CONFIG_SCHEMA
+from .schemas import (
+    SCH_CONFIG_SERIAL,
+    SZ_BLOCK_LIST,
+    SZ_INBOUND,
+    SZ_KNOWN_LIST,
+    SZ_OUTBOUND,
+    SZ_USE_REGEX,
+)
 from .version import VERSION
 
+# TODO: switch dtm from naive to aware
+# TODO: https://evohome-hackers.slack.com/archives/C02SYCLATSL/p1646997554178989
+# TODO: https://evohome-hackers.slack.com/archives/C02SYCLATSL/p1646998253052939
+
+
 # skipcq: PY-W2000
 from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
     I_,
     RP,
     RQ,
     W_,
+    Code,
 )
 
-# skipcq: PY-W2000
-from .const import (  # noqa: F401, isort: skip, pylint: disable=unused-import
-    _0006,
-    _0016,
-    _0404,
-    _0418,
-    _1F09,
-    _1FC9,
-    _3220,
-)
 
 DEV_MODE = __dev_mode__ and False  # debug is_wanted, or qos_fx
 DEV_HACK_REGEX = False
 
+
 _LOGGER = logging.getLogger(__name__)
 if DEV_MODE:  # or True:
     _LOGGER.setLevel(logging.DEBUG)  # should be INFO
 
-SZ_BLOCK_LIST = "block_list"
-SZ_KNOWN_LIST = "known_list"
+
+_DEFAULT_USE_REGEX = {
+    SZ_INBOUND: {"( 03:.* 03:.* (1060|2389|30C9) 003) ..": "\\1 00"},
+    SZ_OUTBOUND: {},
+}
+
 TIP = f", configure the {SZ_KNOWN_LIST}/{SZ_BLOCK_LIST} as required"
 
 IS_INITIALIZED = "is_initialized"
 IS_EVOFW3 = "is_evofw3"
 
-DEFAULT_SERIAL_CONFIG = SERIAL_CONFIG_SCHEMA({})
-
 ERR_MSG_REGEX = re.compile(r"^([0-9A-F]{2}\.)+$")
 
 SZ_POLLER_TASK = "poller_task"
@@ -127,65 +121,8 @@ VALID_CHARACTERS = printable  # "".join((ascii_letters, digits, ":-<*# "))
 # !FS - save autotune
 
 
-class Qos:
-    """The QoS class.
-
-    This is a mess - it is the first step in cleaning up QoS.
-    """
-
-    POLL_INTERVAL = 0.002
-
-    # tx (from sent to gwy, to get back from gwy) seems to takes appDEFAULT_KEYSrox. 0.025s
-    DEFAULT_TX_TIMEOUT = td(seconds=0.2)  # 0.20 OK, but too high?
-    DEFAULT_TX_RETRIES = 2
-    RETRY_LIMIT_MAX = 5
-
-    DEFAULT_RX_TIMEOUT = td(seconds=0.50)  # 0.20 seems OK, 0.10 too low sometimes
-    MAX_BACKOFF_FACTOR = 2  # i.e. tx_timeout 2 ** MAX_BACKOFF
-
-    QOS_KEYS = (SZ_PRIORITY, SZ_RETRIES, SZ_TIMEOUT, SZ_BACKOFF)
-
-    DEFAULT_QOS = (Priority.DEFAULT, DEFAULT_TX_RETRIES, DEFAULT_TX_TIMEOUT, True)
-    DEFAULT_QOS_TABLE = (
-        {  # priority, retries, timeout, (enable_)backoff, c.f. DEFAULT_QOS
-            f"{RQ}|{_0016}": (Priority.HIGH, 5, None, True),
-            f"{RQ}|{_0006}": (Priority.HIGH, 5, None, True),
-            f"{I_}|{_0404}": (Priority.HIGH, 3, td(seconds=0.30), True),  # short Tx
-            f"{RQ}|{_0404}": (Priority.HIGH, 3, td(seconds=1.00), True),
-            f"{W_}|{_0404}": (Priority.HIGH, 3, td(seconds=1.00), True),  # but long Rx
-            f"{RQ}|{_0418}": (Priority.LOW, 3, None, None),
-            f"{RQ}|{_1F09}": (Priority.HIGH, 5, None, True),
-            f"{I_}|{_1FC9}": (Priority.HIGH, 2, td(seconds=1), False),
-            f"{RQ}|{_3220}": (Priority.DEFAULT, 1, td(seconds=1.2), False),
-            f"{W_}|{_3220}": (Priority.HIGH, 3, td(seconds=1.2), False),
-        }
-    )  # The long timeout for the OTB is for total RTT to slave (boiler)
-
-    def __init__(
-        self,
-        *,
-        priority=None,
-        retries=None,
-        timeout=None,
-        backoff=None,
-    ) -> None:
-
-        self.priority = priority if priority is not None else self.DEFAULT_QOS[0]
-        self.retry_limit = retries if retries is not None else self.DEFAULT_QOS[1]
-        self.tx_timeout = self.DEFAULT_TX_TIMEOUT
-        self.rx_timeout = timeout if timeout is not None else self.DEFAULT_QOS[2]
-        self.disable_backoff = not (
-            backoff if backoff is not None else self.DEFAULT_QOS[3]
-        )
-
-    @classmethod  # constructor from verb|code pair
-    def verb_code(cls, verb, code, **kwargs) -> dict:
-        """Constructor to create a QoS based upon the defaults for a verb|code pair."""
-
-        default_qos = cls.DEFAULT_QOS_TABLE.get(f"{verb}|{code}", cls.DEFAULT_QOS)
-        return cls(
-            **{k: kwargs.get(k, default_qos[i]) for i, k in enumerate(cls.QOS_KEYS)}
-        )
+_PacketProtocolT = TypeVar("_PacketProtocolT", bound=asyncio.BaseProtocol)
+_PacketTransportT = TypeVar("_PacketTransportT", bound=asyncio.BaseTransport)
 
 
 def _str(value: bytes) -> str:
@@ -239,10 +176,10 @@ def _regex_hack(pkt_line: str, regex_filters: dict) -> str:
     return result
 
 
-sync_cycles = deque()
+sync_cycles: deque = deque()  # used by @avoid_system_syncs / @track_system_syncs
 
 
-def avoid_system_syncs(fnc) -> Awaitable:
+def avoid_system_syncs(fnc: Callable[..., Awaitable]):
     """Take measures to avoid Tx when any controller is doing a sync cycle."""
 
     DURATION_PKT_GAP = 0.020  # 0.0200 for evohome, or 0.0127 for DTS92
@@ -256,7 +193,7 @@ def avoid_system_syncs(fnc) -> Awaitable:
 
     times_0 = []  # FIXME: remove
 
-    async def wrapper(*args, **kwargs) -> None:
+    async def wrapper(*args, **kwargs):
         global sync_cycles  # skipcq: PYL-W0602
 
         def is_imminent(p):
@@ -291,19 +228,19 @@ def avoid_system_syncs(fnc) -> Awaitable:
     return wrapper
 
 
-def track_system_syncs(fnc) -> Callable:
+def track_system_syncs(fnc: Callable):
     """Track/remember the most recent sync cycle for a controller."""
 
     MAX_SYNCS_TRACKED = 3
 
-    def wrapper(self, pkt, *args, **kwargs) -> None:
+    def wrapper(self, pkt: Packet, *args, **kwargs) -> None:
         global sync_cycles
 
         def is_pending(p):
             """Return True if a sync cycle is still pending (ignores drift)."""
             return p.dtm + td(seconds=int(p.payload[2:6], 16) / 10) > dt_now()
 
-        if pkt.code != _1F09 or pkt.verb != I_ or pkt._len != 3:
+        if pkt.code != Code._1F09 or pkt.verb != I_ or pkt._len != 3:
             return fnc(self, pkt, *args, **kwargs)
 
         sync_cycles = deque(
@@ -319,7 +256,7 @@ def track_system_syncs(fnc) -> Callable:
     return wrapper
 
 
-def limit_duty_cycle(max_duty_cycle: float, time_window: int = 60) -> Callable:
+def limit_duty_cycle(max_duty_cycle: float, time_window: int = 60):
     """Limit the Tx rate to the RF duty cycle regulations (e.g. 1% per hour).
 
     max_duty_cycle: bandwidth available per observation window (%)
@@ -330,13 +267,13 @@ def limit_duty_cycle(max_duty_cycle: float, time_window: int = 60) -> Callable:
     FILL_RATE: float = TX_RATE_AVAIL * max_duty_cycle  # bits per second
     BUCKET_CAPACITY: float = FILL_RATE * time_window
 
-    def decorator(fnc) -> Awaitable:
+    def decorator(fnc: Callable[..., Awaitable]):
         # start with a full bit bucket
         bits_in_bucket: float = BUCKET_CAPACITY
         last_time_bit_added = perf_counter()
 
         @wraps(fnc)
-        async def wrapper(self, packet, *args, **kwargs) -> None:
+        async def wrapper(self, packet: str, *args, **kwargs):
             nonlocal bits_in_bucket
             nonlocal last_time_bit_added
 
@@ -364,7 +301,7 @@ def limit_duty_cycle(max_duty_cycle: float, time_window: int = 60) -> Callable:
     return decorator
 
 
-def limit_transmit_rate(max_tokens: float, time_window: int = 60) -> Callable:
+def limit_transmit_rate(max_tokens: float, time_window: int = 60):
     """Limit the Tx rate as # packets per period of time.
 
     Rate-limits the decorated function locally, for one process (Token Bucket).
@@ -377,7 +314,7 @@ def limit_transmit_rate(max_tokens: float, time_window: int = 60) -> Callable:
 
     token_fill_rate: float = max_tokens / time_window
 
-    def decorator(fnc) -> Awaitable:
+    def decorator(fnc: Callable):
         token_bucket: float = max_tokens  # initialize with max tokens
         last_time_token_added = perf_counter()
 
@@ -409,38 +346,44 @@ def limit_transmit_rate(max_tokens: float, time_window: int = 60) -> Callable:
 class SerTransportBase(asyncio.ReadTransport):
     """Interface for a packet transport."""
 
+    _extra: dict
+
     def __init__(self, loop, extra=None):
         super().__init__(extra=extra)
 
         self._loop = loop
         self._extra[SZ_POLLER_TASK] = self._loop.create_task(self._polling_loop())
 
-        self._closing = False
-
         # for sig in (signal.SIGINT, signal.SIGTERM):
         #     self._loop.add_signal_handler(sig, self.abort)
 
-    def is_closing(self) -> bool:
-        """Return True if the transport is closing or closed."""
-        return self._closing
-
-    def close(self):
-        if self._closing:
-            return
-
-        if poller := self._extra.get(SZ_POLLER_TASK):
-            poller.cancel()
-
-        self._loop.call_soon(self._protocol.connection_lost, None)
+        self._is_closing = False
 
     async def _polling_loop(self):
         raise NotImplementedError
 
+    def close(self):
+        """Close the transport."""
+
+        if self._is_closing:
+            return
+        self._is_closing = True
+
+        self._protocol.pause_writing()
+        if task := self._extra.get(SZ_POLLER_TASK):
+            task.cancel()
+
+        self._loop.call_soon(self._protocol.connection_lost, None)
+
+    def is_closing(self) -> bool:
+        """Return True if the transport is closing or closed."""
+        return self._is_closing
+
 
 class SerTransportRead(SerTransportBase):
     """Interface for a packet transport via a dict (saved state) or a file (pkt log)."""
 
-    def __init__(self, loop, protocol, packet_source, extra=None):
+    def __init__(self, loop, protocol: PacketProtocolBase, packet_source, extra=None):
         super().__init__(loop, extra=extra)
 
         self._protocol = protocol
@@ -475,14 +418,14 @@ class SerTransportPoll(SerTransportBase):
 
     MAX_BUFFER_SIZE = 500
 
-    def __init__(self, loop, protocol, ser_instance, extra=None):
+    def __init__(self, loop, protocol: PacketProtocolBase, ser_instance, extra=None):
         super().__init__(loop, extra=extra)
 
         self._protocol = protocol
         self.serial = ser_instance
 
         self._is_closing = None
-        self._write_queue = Queue(maxsize=self.MAX_BUFFER_SIZE)
+        self._write_queue: Queue = Queue(maxsize=self.MAX_BUFFER_SIZE)
 
     async def _polling_loop(self):
         self._protocol.connection_made(self)
@@ -528,17 +471,17 @@ class PacketProtocolBase(asyncio.Protocol):
 
         self._gwy = gwy
         self._loop = gwy._loop
-        self._callback = pkt_handler  # Could be None
+        self._callback: None | Callable = pkt_handler
 
-        self._transport = None
+        self._transport: asyncio.Transport = None  # type: ignore[assignment]
         self._pause_writing = True
         self._recv_buffer = bytes()
 
-        self._prev_pkt = None
-        self._this_pkt = None
+        self._prev_pkt: Packet = None  # type: ignore[assignment]
+        self._this_pkt: Packet = None  # type: ignore[assignment]
 
         self._disable_sending = gwy.config.disable_sending
-        self._evofw_flag = gwy.config.evofw_flag
+        self._evofw_flag = getattr(gwy.config, "evofw_flag", None)
 
         self.enforce_include = gwy.config.enforce_known_list
         if self.enforce_include:
@@ -550,10 +493,12 @@ class PacketProtocolBase(asyncio.Protocol):
         self._unwanted: list = []  # not: [NON_DEV_ADDR.id, NUL_DEV_ADDR.id]
 
         self._hgi80 = {
-            IS_INITIALIZED: None,
+            IS_INITIALIZED: False,
             IS_EVOFW3: None,
-            SZ_DEVICE_ID: None,
-        }
+            SZ_DEVICE_ID: HGI_DEV_ADDR.id,
+        }  # also: "evofw3_ver"
+
+        self._use_regex = getattr(self._gwy.config, SZ_USE_REGEX, {})
 
     def __repr__(self) -> str:
         return f"{self.__class__.__name__}(enforce_include={self.enforce_include})"
@@ -561,8 +506,7 @@ class PacketProtocolBase(asyncio.Protocol):
     def __str__(self) -> str:
         return self.__class__.__name__
 
-    @staticmethod
-    def _dt_now() -> dt:
+    def _dt_now(self) -> dt:
         """Return a precise datetime, using the curent dtm."""
         return dt_now()
 
@@ -611,7 +555,7 @@ class PacketProtocolBase(asyncio.Protocol):
     def data_received(self, data: bytes) -> None:
         """Called by the transport when some data (packet fragments) is received."""
 
-        def bytes_received(data: bytes) -> Iterable[bytes]:
+        def bytes_received(data: bytes) -> Iterable[tuple[dt, bytes]]:
             self._recv_buffer += data
             if b"\r\n" in self._recv_buffer:
                 lines = self._recv_buffer.split(b"\r\n")
@@ -626,7 +570,7 @@ class PacketProtocolBase(asyncio.Protocol):
         """Check/set HGI; log if it is a foreign HGI."""
         assert pkt.src.type == DEV_TYPE_MAP.HGI and pkt.src.id != HGI_DEV_ADDR.id
 
-        if self._hgi80[SZ_DEVICE_ID] is None:
+        if self._hgi80[SZ_DEVICE_ID] is HGI_DEV_ADDR.id:
             self._hgi80[SZ_DEVICE_ID] = pkt.src.id
 
         elif (
@@ -648,17 +592,16 @@ class PacketProtocolBase(asyncio.Protocol):
             pkt = Packet.from_port(
                 self._gwy,
                 dtm,
-                _regex_hack(line, self._gwy.config.use_regex.get(SZ_INBOUND, {})),
+                _regex_hack(line, self._use_regex.get(SZ_INBOUND, {})),
                 raw_line=raw_line,
             )  # should log all? invalid pkts appropriately
 
-            if (
-                pkt.src.type == DEV_TYPE_MAP.HGI
-            ):  # dex: ideally should use HGI, but how?
+            if pkt.src.type == DEV_TYPE_MAP.HGI:
                 self._check_set_hgi80(pkt)
         except InvalidPacketError as exc:
             if "# evofw" in line and self._hgi80[IS_EVOFW3] is None:
-                self._hgi80[IS_EVOFW3] = line
+                self._hgi80[IS_EVOFW3] = True
+                self._hgi80["evofw3_ver"] = line
                 if self._evofw_flag not in (None, "!V"):
                     self._transport.write(
                         bytes(f"{self._evofw_flag}\r\n".encode("ascii"))
@@ -717,7 +660,7 @@ class PacketProtocolBase(asyncio.Protocol):
                 self._include.append(dev_id)  # NOTE: only time include list is modified
                 continue
 
-            if dev_id[:2] == DEV_TYPE_MAP.HGI and self._hgi80[SZ_DEVICE_ID] is None:
+            if dev_id[:2] == DEV_TYPE_MAP.HGI and not self._hgi80[SZ_DEVICE_ID]:
                 _LOGGER.debug(f"Allowed {dev_id} (is a gateway?){TIP}")
                 continue
 
@@ -748,7 +691,7 @@ class PacketProtocolFile(PacketProtocolBase):
     def __init__(self, gwy, pkt_handler: Callable) -> None:
         super().__init__(gwy, pkt_handler)
 
-        self._dt_str_ = None
+        self._dt_str_: str = None  # type: ignore[assignment]
 
     def _dt_now(self) -> dt:
         """Return a precise datetime, using a packet's dtm field."""
@@ -763,20 +706,20 @@ class PacketProtocolFile(PacketProtocolBase):
         except AttributeError:
             return dt(1970, 1, 1, 1, 0)
 
-    def data_received(self, data: str) -> None:
+    def data_received(self, data: str) -> None:  # type: ignore[override]
         """Called when a packet line is received (from a log file)."""
 
         self._dt_str_ = data[:26]  # used for self._dt_now
 
         self._line_received(data[:26], data[27:].strip(), data)
 
-    def _line_received(self, dtm: str, line: str, raw_line: str) -> None:
+    def _line_received(self, dtm: str, line: str, raw_line: str) -> None:  # type: ignore[override]
 
         try:
             pkt = Packet.from_file(
                 self._gwy,
                 dtm,
-                _regex_hack(line, self._gwy.config.use_regex.get(SZ_INBOUND, {})),
+                _regex_hack(line, self._use_regex.get(SZ_INBOUND, {})),
             )  # should log all invalid pkts appropriately
 
         except (InvalidPacketError, ValueError):  # VE from dt.fromisoformat()
@@ -798,10 +741,6 @@ class PacketProtocolPort(PacketProtocolBase):
         self._sem = asyncio.BoundedSemaphore()
         self._leaker = None
 
-        # for sig in (signal.SIGINT, signal.SIGTERM):
-        #     self._loop.add_signal_handler(sig, self._shutdown)
-        pass
-
     async def _leak_sem(self):
         """Used to enforce a minimum time between calls to `self._transport.write()`."""
         while True:
@@ -877,10 +816,10 @@ class PacketProtocolPort(PacketProtocolBase):
         # ):
         #     await asyncio.sleep(0.005)
 
-        data = bytes(
+        data_bytes = bytes(
             _regex_hack(
                 data,
-                self._gwy.config.use_regex.get(SZ_OUTBOUND, {}),
+                self._use_regex.get(SZ_OUTBOUND, {}),
             ).encode("ascii")
         )
 
@@ -888,7 +827,7 @@ class PacketProtocolPort(PacketProtocolBase):
 
         if _LOGGER.getEffectiveLevel() == logging.INFO:  # i.e. don't log for DEBUG
             _LOGGER.info("RF Tx:     %s", data)
-        self._transport.write(data + b"\r\n")
+        self._transport.write(data_bytes + b"\r\n")
 
 
 class PacketProtocolQos(PacketProtocolPort):
@@ -898,9 +837,9 @@ class PacketProtocolQos(PacketProtocolPort):
         super().__init__(gwy, pkt_handler)
 
         # self._qos_lock = Lock()
-        self._qos_cmd = None
-        self._tx_rcvd = None
-        self._rx_rcvd = None
+        self._qos_cmd: None | Command = None
+        self._tx_rcvd: None | Packet = None
+        self._rx_rcvd: None | Packet = None
 
     def _pkt_received(self, pkt: Packet) -> None:
         """Called when packets are received (a callback).
@@ -922,7 +861,7 @@ class PacketProtocolQos(PacketProtocolPort):
 
         super()._pkt_received(pkt)
 
-    async def send_data(self, cmd: Command) -> Optional[Packet]:
+    async def send_data(self, cmd: Command) -> Optional[Packet]:  # type: ignore[override]
         """Called when packets are to be sent (not a callback)."""
 
         if self._disable_sending:
@@ -989,14 +928,14 @@ class PacketProtocolQos(PacketProtocolPort):
 
 def create_pkt_stack(
     gwy,
-    pkt_callback,
+    pkt_callback: Callable,
     /,
     *,
-    protocol_factory=None,
-    ser_port=None,
+    protocol_factory: Callable = None,
+    ser_port: str = None,
     packet_log=None,
-    packet_dict=None,
-) -> tuple[asyncio.Protocol, asyncio.Transport]:
+    packet_dict: dict = None,
+) -> tuple[_PacketProtocolT, _PacketTransportT]:
     """Utility function to provide a transport to the internal protocol.
 
     The architecture is: app (client) -> msg -> pkt -> ser (HW interface) / log (file).
@@ -1006,13 +945,13 @@ def create_pkt_stack(
      - MsgTransport.write (pkt_dispatcher) to (pkt_protocol) PktProtocol.send_data
     """
 
-    def get_serial_instance(ser_name, ser_config) -> SerialBase:
+    def get_serial_instance(ser_name: str, ser_config: dict) -> SerialBase:
 
         # For example:
         # - python client.py monitor 'rfc2217://localhost:5001'
         # - python client.py monitor 'alt:///dev/ttyUSB0?class=PosixPollSerial'
 
-        ser_config = {**DEFAULT_SERIAL_CONFIG, **ser_config}
+        ser_config = SCH_CONFIG_SERIAL(ser_config)
 
         try:
             ser_obj = serial_for_url(ser_name, **ser_config)
@@ -1033,7 +972,7 @@ def create_pkt_stack(
 
         return ser_obj
 
-    def issue_warning():
+    def issue_warning() -> None:
         _LOGGER.warning(
             f"{'Windows' if os.name == 'nt' else 'This type of serial interface'} "
             "is not fully supported by this library: "
@@ -1042,7 +981,7 @@ def create_pkt_stack(
             "(e.g. linux with a local serial port)"
         )
 
-    def protocol_factory_():
+    def protocol_factory_() -> type[_PacketProtocolT]:
         if packet_log or packet_dict is not None:
             return create_protocol_factory(PacketProtocolFile, gwy, pkt_callback)()
         elif gwy.config.disable_sending:  # NOTE: assumes we wont change our mind
@@ -1055,18 +994,15 @@ def create_pkt_stack(
     if len([x for x in (packet_dict, packet_log, ser_port) if x is not None]) != 1:
         raise TypeError("serial port, log file & dict should be mutually exclusive")
 
-    ser_instance = (
-        get_serial_instance(ser_port, gwy.config.serial_config) if ser_port else None
-    )  # do this first, in case raises a SerialException
-
     pkt_protocol = (protocol_factory or protocol_factory_)()
 
     if (pkt_source := packet_log or packet_dict) is not None:  # {} is a processable log
-        pkt_transport = SerTransportRead(gwy._loop, pkt_protocol, pkt_source)
-    elif os.name == "nt" or ser_port[:8] == "rfc2217:" or ser_port[:7] == "socket:":
-        issue_warning()
-        pkt_transport = SerTransportPoll(gwy._loop, pkt_protocol, ser_instance)
-    else:  # use the standard serial_asyncio library
-        pkt_transport = SerTransportAsync(gwy._loop, pkt_protocol, ser_instance)
+        return pkt_protocol, SerTransportRead(gwy._loop, pkt_protocol, pkt_source)
 
-    return (pkt_protocol, pkt_transport)
+    ser_instance = get_serial_instance(ser_port, gwy._port_config)
+
+    if os.name == "nt" or ser_instance.portstr[:7] in ("rfc2217", "socket:"):
+        issue_warning()
+        return pkt_protocol, SerTransportPoll(gwy._loop, pkt_protocol, ser_instance)
+
+    return pkt_protocol, SerTransportAsync(gwy._loop, pkt_protocol, ser_instance)
